@@ -9,7 +9,6 @@ import (
 
 	"github.com/containers/podman/v3/pkg/annotations"
 	"github.com/containers/podman/v3/pkg/hooks"
-	"github.com/containers/podman/v3/pkg/registrar"
 	cstorage "github.com/containers/storage"
 	"github.com/containers/storage/pkg/ioutils"
 	"github.com/containers/storage/pkg/truncindex"
@@ -17,6 +16,7 @@ import (
 	"github.com/cri-o/cri-o/internal/lib/sandbox"
 	"github.com/cri-o/cri-o/internal/log"
 	"github.com/cri-o/cri-o/internal/oci"
+	"github.com/cri-o/cri-o/internal/registrar"
 	"github.com/cri-o/cri-o/internal/storage"
 	crioann "github.com/cri-o/cri-o/pkg/annotations"
 	libconfig "github.com/cri-o/cri-o/pkg/config"
@@ -205,7 +205,7 @@ func (c *ContainerServer) LoadSandbox(ctx context.Context, id string) (sb *sandb
 	defer func() {
 		if retErr != nil {
 			if err := sb.RemoveManagedNamespaces(); err != nil {
-				log.Warnf(ctx, "failed to remove namespaces: %v", err)
+				log.Warnf(ctx, "Failed to remove namespaces: %v", err)
 			}
 		}
 	}()
@@ -216,7 +216,7 @@ func (c *ContainerServer) LoadSandbox(ctx context.Context, id string) (sb *sandb
 	defer func() {
 		if retErr != nil {
 			if err := c.RemoveSandbox(sb.ID()); err != nil {
-				log.Warnf(ctx, "could not remove sandbox ID %s: %v", sb.ID(), err)
+				log.Warnf(ctx, "Could not remove sandbox ID %s: %v", sb.ID(), err)
 			}
 		}
 	}()
@@ -257,21 +257,21 @@ func (c *ContainerServer) LoadSandbox(ctx context.Context, id string) (sb *sandb
 		if err != nil {
 			return sb, err
 		}
-		scontainer.SetSpec(&m)
-		scontainer.SetMountPoint(m.Annotations[annotations.MountPoint])
-
-		if m.Annotations[annotations.Volumes] != "" {
-			containerVolumes := []oci.ContainerVolume{}
-			if err = json.Unmarshal([]byte(m.Annotations[annotations.Volumes]), &containerVolumes); err != nil {
-				return sb, fmt.Errorf("failed to unmarshal container volumes: %v", err)
-			}
-			for _, cv := range containerVolumes {
-				scontainer.AddVolume(cv)
-			}
-		}
 	} else {
 		scontainer = oci.NewSpoofedContainer(cID, cname, labels, id, created, sandboxPath)
 	}
+	scontainer.SetSpec(&m)
+	scontainer.SetMountPoint(m.Annotations[annotations.MountPoint])
+
+	if m.Annotations[annotations.Volumes] != "" {
+		containerVolumes := []oci.ContainerVolume{}
+		if err = json.Unmarshal([]byte(m.Annotations[annotations.Volumes]), &containerVolumes); err != nil {
+			return sb, fmt.Errorf("failed to unmarshal container volumes: %v", err)
+		}
+		for _, cv := range containerVolumes {
+			scontainer.AddVolume(cv)
+		}
+	}
 
 	if err := sb.SetInfraContainer(scontainer); err != nil {
 		return sb, err
@@ -319,7 +319,7 @@ func (c *ContainerServer) LoadSandbox(ctx context.Context, id string) (sb *sandb
 	defer func() {
 		if retErr != nil {
 			if err1 := c.ctrIDIndex.Delete(scontainer.ID()); err1 != nil {
-				log.Warnf(ctx, "could not delete container ID %s: %v", scontainer.ID(), err1)
+				log.Warnf(ctx, "Could not delete container ID %s: %v", scontainer.ID(), err1)
 			}
 		}
 	}()
@@ -473,7 +473,7 @@ func (c *ContainerServer) ContainerStateFromDisk(ctx context.Context, ctr *oci.C
 // on disk
 func (c *ContainerServer) ContainerStateToDisk(ctx context.Context, ctr *oci.Container) error {
 	if err := c.Runtime().UpdateContainerStatus(ctx, ctr); err != nil {
-		log.Warnf(ctx, "error updating the container status %q: %v", ctr.ID(), err)
+		log.Warnf(ctx, "Error updating the container status %q: %v", ctr.ID(), err)
 	}
 
 	jsonSource, err := ioutils.NewAtomicFileWriter(ctr.StatePath(), 0o644)
@@ -495,6 +495,11 @@ func (c *ContainerServer) ReserveContainerName(id, name string) (string, error)
 	return name, nil
 }
 
+// ContainerIDForName gets the container ID given the container name from the ID Index
+func (c *ContainerServer) ContainerIDForName(name string) (string, error) {
+	return c.ctrNameIndex.Get(name)
+}
+
 // ReleaseContainerName releases a container name from the index so that it can
 // be used by other containers
 func (c *ContainerServer) ReleaseContainerName(name string) {
@@ -517,6 +522,11 @@ func (c *ContainerServer) ReleasePodName(name string) {
 	c.podNameIndex.Release(name)
 }
 
+// PodIDForName gets the pod ID given the pod name from the ID Index
+func (c *ContainerServer) PodIDForName(name string) (string, error) {
+	return c.podNameIndex.Get(name)
+}
+
 // recoverLogError recovers a runtime panic and logs the returned error if
 // existing
 func recoverLogError() {
