@@ -5,12 +5,10 @@ import (
 	"fmt"
 	"time"
 
-	"github.com/containers/image/v5/copy"
 	istorage "github.com/containers/image/v5/storage"
 	"github.com/containers/image/v5/transports/alltransports"
 	"github.com/containers/image/v5/types"
 	"github.com/containers/storage"
-	"github.com/containers/storage/pkg/idtools"
 	json "github.com/json-iterator/go"
 	v1 "github.com/opencontainers/image-spec/specs-go/v1"
 	"github.com/pkg/errors"
@@ -72,12 +70,7 @@ type RuntimeServer interface {
 	// both its pod's ID and its container ID.
 	// Pointer arguments can be nil.  Either the image name or ID can be
 	// omitted, but not both.  All other arguments are required.
-	CreatePodSandbox(systemContext *types.SystemContext, podName, podID, imageName, imageAuthFile, imageID, containerName, metadataName, uid, namespace string, attempt uint32, idMappings *idtools.IDMappings, labelOptions []string, privileged bool) (ContainerInfo, error)
-	// RemovePodSandbox deletes a pod sandbox's infrastructure container.
-	// The CRI expects that a sandbox can't be removed unless its only
-	// container is its infrastructure container, but we don't enforce that
-	// here, since we're just keeping track of it for higher level APIs.
-	RemovePodSandbox(idOrName string) error
+	CreatePodSandbox(systemContext *types.SystemContext, podName, podID, imageName, imageAuthFile, imageID, containerName, metadataName, uid, namespace string, attempt uint32, idMappingsOptions *storage.IDMappingOptions, labelOptions []string, privileged bool) (ContainerInfo, error)
 
 	// GetContainerMetadata returns the metadata we've stored for a container.
 	GetContainerMetadata(idOrName string) (RuntimeContainerMetadata, error)
@@ -87,7 +80,7 @@ type RuntimeServer interface {
 	// CreateContainer creates a container with the specified ID.
 	// Pointer arguments can be nil.  Either the image name or ID can be
 	// omitted, but not both.  All other arguments are required.
-	CreateContainer(systemContext *types.SystemContext, podName, podID, imageName, imageID, containerName, containerID, metadataName string, attempt uint32, idMappings *idtools.IDMappings, labelOptions []string, privileged bool) (ContainerInfo, error)
+	CreateContainer(systemContext *types.SystemContext, podName, podID, imageName, imageID, containerName, containerID, metadataName string, attempt uint32, idMappingsOptions *storage.IDMappingOptions, labelOptions []string, privileged bool) (ContainerInfo, error)
 	// DeleteContainer deletes a container, unmounting it first if need be.
 	DeleteContainer(idOrName string) error
 
@@ -148,7 +141,7 @@ func (metadata *RuntimeContainerMetadata) SetMountLabel(mountLabel string) {
 	metadata.MountLabel = mountLabel
 }
 
-func (r *runtimeService) createContainerOrPodSandbox(systemContext *types.SystemContext, podName, podID, imageName, imageAuthFile, imageID, containerName, containerID, metadataName, uid, namespace string, attempt uint32, idMappings *idtools.IDMappings, labelOptions []string, isPauseImage, privileged bool) (ci ContainerInfo, retErr error) {
+func (r *runtimeService) createContainerOrPodSandbox(systemContext *types.SystemContext, podName, podID, imageName, imageAuthFile, imageID, containerName, containerID, metadataName, uid, namespace string, attempt uint32, idMappingsOptions *storage.IDMappingOptions, labelOptions []string, isPauseImage, privileged bool) (ci ContainerInfo, retErr error) {
 	var ref types.ImageReference
 	if podName == "" || podID == "" {
 		return ContainerInfo{}, ErrInvalidPodName
@@ -188,7 +181,7 @@ func (r *runtimeService) createContainerOrPodSandbox(systemContext *types.System
 		if image == "" {
 			return ContainerInfo{}, ErrInvalidImageName
 		}
-		logrus.Debugf("couldn't find image %q, retrieving it", image)
+		logrus.Debugf("Couldn't find image %q, retrieving it", image)
 		sourceCtx := types.SystemContext{}
 		if systemContext != nil {
 			sourceCtx = *systemContext // A shallow copy
@@ -196,7 +189,7 @@ func (r *runtimeService) createContainerOrPodSandbox(systemContext *types.System
 		if imageAuthFile != "" {
 			sourceCtx.AuthFilePath = imageAuthFile
 		}
-		ref, err = r.storageImageServer.PullImage(systemContext, image, &copy.Options{
+		ref, err = r.storageImageServer.PullImage(systemContext, image, &ImageCopyOptions{
 			SourceCtx:      &sourceCtx,
 			DestinationCtx: systemContext,
 		})
@@ -207,7 +200,7 @@ func (r *runtimeService) createContainerOrPodSandbox(systemContext *types.System
 		if err != nil {
 			return ContainerInfo{}, err
 		}
-		logrus.Debugf("successfully pulled image %q", image)
+		logrus.Debugf("Successfully pulled image %q", image)
 	}
 	if img == nil && errors.Is(err, storage.ErrImageUnknown) {
 		if imageID == "" {
@@ -265,23 +258,27 @@ func (r *runtimeService) createContainerOrPodSandbox(systemContext *types.System
 
 	coptions := storage.ContainerOptions{
 		LabelOpts: labelOptions,
+		Volatile:  true,
 	}
-	if idMappings != nil {
-		coptions.IDMappingOptions = storage.IDMappingOptions{UIDMap: idMappings.UIDs(), GIDMap: idMappings.GIDs()}
+	if idMappingsOptions != nil {
+		coptions.IDMappingOptions = *idMappingsOptions
 	}
 	container, err := r.storageImageServer.GetStore().CreateContainer(containerID, names, img.ID, "", string(mdata), &coptions)
 	if err != nil {
 		if metadata.Pod {
-			logrus.Debugf("failed to create pod sandbox %s(%s): %v", metadata.PodName, metadata.PodID, err)
+			logrus.Debugf("Failed to create pod sandbox %s(%s): %v", metadata.PodName, metadata.PodID, err)
 		} else {
-			logrus.Debugf("failed to create container %s(%s): %v", metadata.ContainerName, containerID, err)
+			logrus.Debugf("Failed to create container %s(%s): %v", metadata.ContainerName, containerID, err)
 		}
 		return ContainerInfo{}, err
 	}
+	if idMappingsOptions != nil {
+		*idMappingsOptions = coptions.IDMappingOptions
+	}
 	if metadata.Pod {
-		logrus.Debugf("created pod sandbox %q", container.ID)
+		logrus.Debugf("Created pod sandbox %q", container.ID)
 	} else {
-		logrus.Debugf("created container %q", container.ID)
+		logrus.Debugf("Created container %q", container.ID)
 	}
 
 	// If anything fails after this point, we need to delete the incomplete
@@ -296,7 +293,7 @@ func (r *runtimeService) createContainerOrPodSandbox(systemContext *types.System
 				}
 				return
 			}
-			logrus.Debugf("deleted partially-created container %q", container.ID)
+			logrus.Debugf("Deleted partially-created container %q", container.ID)
 		}
 	}()
 
@@ -319,9 +316,9 @@ func (r *runtimeService) createContainerOrPodSandbox(systemContext *types.System
 		return ContainerInfo{}, err
 	}
 	if metadata.Pod {
-		logrus.Debugf("pod sandbox %q has work directory %q", container.ID, containerDir)
+		logrus.Debugf("Pod sandbox %q has work directory %q", container.ID, containerDir)
 	} else {
-		logrus.Debugf("container %q has work directory %q", container.ID, containerDir)
+		logrus.Debugf("Container %q has work directory %q", container.ID, containerDir)
 	}
 
 	containerRunDir, err := r.storageImageServer.GetStore().ContainerRunDirectory(container.ID)
@@ -329,9 +326,9 @@ func (r *runtimeService) createContainerOrPodSandbox(systemContext *types.System
 		return ContainerInfo{}, err
 	}
 	if metadata.Pod {
-		logrus.Debugf("pod sandbox %q has run directory %q", container.ID, containerRunDir)
+		logrus.Debugf("Pod sandbox %q has run directory %q", container.ID, containerRunDir)
 	} else {
-		logrus.Debugf("container %q has run directory %q", container.ID, containerRunDir)
+		logrus.Debugf("Container %q has run directory %q", container.ID, containerRunDir)
 	}
 
 	metadata.MountLabel = container.MountLabel()
@@ -346,28 +343,39 @@ func (r *runtimeService) createContainerOrPodSandbox(systemContext *types.System
 	}, nil
 }
 
-func (r *runtimeService) CreatePodSandbox(systemContext *types.SystemContext, podName, podID, imageName, imageAuthFile, imageID, containerName, metadataName, uid, namespace string, attempt uint32, idMappings *idtools.IDMappings, labelOptions []string, privileged bool) (ContainerInfo, error) {
-	return r.createContainerOrPodSandbox(systemContext, podName, podID, imageName, imageAuthFile, imageID, containerName, podID, metadataName, uid, namespace, attempt, idMappings, labelOptions, true, privileged)
+func (r *runtimeService) CreatePodSandbox(systemContext *types.SystemContext, podName, podID, imageName, imageAuthFile, imageID, containerName, metadataName, uid, namespace string, attempt uint32, idMappingsOptions *storage.IDMappingOptions, labelOptions []string, privileged bool) (ContainerInfo, error) {
+	return r.createContainerOrPodSandbox(systemContext, podName, podID, imageName, imageAuthFile, imageID, containerName, podID, metadataName, uid, namespace, attempt, idMappingsOptions, labelOptions, true, privileged)
 }
 
-func (r *runtimeService) CreateContainer(systemContext *types.SystemContext, podName, podID, imageName, imageID, containerName, containerID, metadataName string, attempt uint32, idMappings *idtools.IDMappings, labelOptions []string, privileged bool) (ContainerInfo, error) {
-	return r.createContainerOrPodSandbox(systemContext, podName, podID, imageName, "", imageID, containerName, containerID, metadataName, "", "", attempt, idMappings, labelOptions, false, privileged)
+func (r *runtimeService) CreateContainer(systemContext *types.SystemContext, podName, podID, imageName, imageID, containerName, containerID, metadataName string, attempt uint32, idMappingsOptions *storage.IDMappingOptions, labelOptions []string, privileged bool) (ContainerInfo, error) {
+	return r.createContainerOrPodSandbox(systemContext, podName, podID, imageName, "", imageID, containerName, containerID, metadataName, "", "", attempt, idMappingsOptions, labelOptions, false, privileged)
 }
 
-func (r *runtimeService) RemovePodSandbox(idOrName string) error {
-	container, err := r.storageImageServer.GetStore().Container(idOrName)
+func (r *runtimeService) deleteLayerIfMapped(imageID, layerID string) {
+	if layerID == "" {
+		return
+	}
+	store := r.storageImageServer.GetStore()
+
+	image, err := store.Image(imageID)
 	if err != nil {
-		if errors.Is(err, storage.ErrContainerUnknown) {
-			return ErrInvalidSandboxID
+		logrus.Debugf("Failed to retrieve image %q: %v", imageID, err)
+		return
+	}
+
+	// ignore if it is the top layer.  It was pulled already with the specified
+	// mapping.  In this case we don't delete it.
+	if image.TopLayer == layerID {
+		return
+	}
+	for _, ml := range image.MappedTopLayers {
+		if ml == layerID {
+			// if the layer is used by other containers, DeleteLayer
+			// will fail.
+			store.DeleteLayer(layerID) // nolint: errcheck
+			return
 		}
-		return err
 	}
-	err = r.storageImageServer.GetStore().DeleteContainer(container.ID)
-	if err != nil {
-		logrus.Debugf("failed to delete pod sandbox %q: %v", container.ID, err)
-		return err
-	}
-	return nil
 }
 
 func (r *runtimeService) DeleteContainer(idOrName string) error {
@@ -375,21 +383,32 @@ func (r *runtimeService) DeleteContainer(idOrName string) error {
 		return ErrInvalidContainerID
 	}
 	container, err := r.storageImageServer.GetStore().Container(idOrName)
+	// Already deleted
+	if errors.Is(err, storage.ErrContainerUnknown) {
+		return nil
+	}
 	if err != nil {
 		return err
 	}
+	layer, err := r.storageImageServer.GetStore().Layer(container.LayerID)
+	if err != nil {
+		logrus.Debugf("Failed to retrieve layer %q: %v", container.LayerID, err)
+	}
 	err = r.storageImageServer.GetStore().DeleteContainer(container.ID)
 	if err != nil {
-		logrus.Debugf("failed to delete container %q: %v", container.ID, err)
+		logrus.Debugf("Failed to delete container %q: %v", container.ID, err)
 		return err
 	}
+	if layer != nil {
+		r.deleteLayerIfMapped(container.ImageID, layer.Parent)
+	}
 	return nil
 }
 
 func (r *runtimeService) SetContainerMetadata(idOrName string, metadata *RuntimeContainerMetadata) error {
 	mdata, err := json.Marshal(&metadata)
 	if err != nil {
-		logrus.Debugf("failed to encode metadata for %q: %v", idOrName, err)
+		logrus.Debugf("Failed to encode metadata for %q: %v", idOrName, err)
 		return err
 	}
 	return r.storageImageServer.GetStore().SetMetadata(idOrName, string(mdata))
@@ -421,10 +440,10 @@ func (r *runtimeService) StartContainer(idOrName string) (string, error) {
 	}
 	mountPoint, err := r.storageImageServer.GetStore().Mount(container.ID, metadata.MountLabel)
 	if err != nil {
-		logrus.Debugf("failed to mount container %q: %v", container.ID, err)
+		logrus.Debugf("Failed to mount container %q: %v", container.ID, err)
 		return "", err
 	}
-	logrus.Debugf("mounted container %q at %q", container.ID, mountPoint)
+	logrus.Debugf("Mounted container %q at %q", container.ID, mountPoint)
 	return mountPoint, nil
 }
 
@@ -438,10 +457,10 @@ func (r *runtimeService) StopContainer(idOrName string) error {
 	}
 	_, err = r.storageImageServer.GetStore().Unmount(container.ID, false)
 	if err != nil {
-		logrus.Debugf("failed to unmount container %q: %v", container.ID, err)
+		logrus.Debugf("Failed to unmount container %q: %v", container.ID, err)
 		return err
 	}
-	logrus.Debugf("unmounted container %q", container.ID)
+	logrus.Debugf("Unmounted container %q", container.ID)
 	return nil
 }
 
