@@ -19,6 +19,7 @@ import (
 	"github.com/containers/podman/v3/pkg/rootless"
 	"github.com/containers/storage"
 	"github.com/cri-o/cri-o/internal/config/apparmor"
+	"github.com/cri-o/cri-o/internal/config/blockio"
 	"github.com/cri-o/cri-o/internal/config/capabilities"
 	"github.com/cri-o/cri-o/internal/config/cgmgr"
 	"github.com/cri-o/cri-o/internal/config/cnimgr"
@@ -26,9 +27,11 @@ import (
 	"github.com/cri-o/cri-o/internal/config/device"
 	"github.com/cri-o/cri-o/internal/config/node"
 	"github.com/cri-o/cri-o/internal/config/nsmgr"
+	"github.com/cri-o/cri-o/internal/config/rdt"
 	"github.com/cri-o/cri-o/internal/config/seccomp"
 	"github.com/cri-o/cri-o/internal/config/ulimits"
 	"github.com/cri-o/cri-o/pkg/annotations"
+	"github.com/cri-o/cri-o/server/metrics/collectors"
 	"github.com/cri-o/cri-o/server/useragent"
 	"github.com/cri-o/cri-o/utils"
 	"github.com/cri-o/cri-o/utils/cmdrunner"
@@ -114,6 +117,11 @@ const (
 	DefaultLogSizeMax = -1
 )
 
+const (
+	// DefaultBlockIOConfigFile is the default value for blockio controller configuration file
+	DefaultBlockIOConfigFile = ""
+)
+
 const (
 	// DefaultIrqBalanceConfigFile default irqbalance service configuration file path
 	DefaultIrqBalanceConfigFile = "/etc/sysconfig/irqbalance"
@@ -158,15 +166,18 @@ type RootConfig struct {
 
 	// InternalWipe is whether CRI-O should wipe containers and images after a reboot when the server starts.
 	// If set to false, one must use the external command `crio wipe` to wipe the containers and images in these situations.
+	// The option InternalWipe is deprecated, and will be removed in a future release.
 	InternalWipe bool `toml:"internal_wipe"`
 }
 
 // RuntimeHandler represents each item of the "crio.runtime.runtimes" TOML
 // config table.
 type RuntimeHandler struct {
-	RuntimePath string `toml:"runtime_path"`
-	RuntimeType string `toml:"runtime_type"`
-	RuntimeRoot string `toml:"runtime_root"`
+	RuntimeConfigPath string `toml:"runtime_config_path"`
+	RuntimePath       string `toml:"runtime_path"`
+	RuntimeType       string `toml:"runtime_type"`
+	RuntimeRoot       string `toml:"runtime_root"`
+
 	// PrivilegedWithoutHostDevices can be used to restrict passing host devices
 	// to a container running as privileged.
 	PrivilegedWithoutHostDevices bool `toml:"privileged_without_host_devices,omitempty"`
@@ -237,6 +248,10 @@ type RuntimeConfig struct {
 	// Devices to add to containers
 	AdditionalDevices []string `toml:"additional_devices"`
 
+	// DeviceOwnershipFromSecurityContext changes the default behavior of setting container devices uid/gid
+	// from CRI's SecurityContext (RunAsUser/RunAsGroup) instead of taking host's uid/gid. Defaults to false.
+	DeviceOwnershipFromSecurityContext bool `toml:"device_ownership_from_security_context"`
+
 	// DefaultRuntime is the _name_ of the OCI runtime to be used as the default.
 	// The name is matched against the Runtimes map below.
 	DefaultRuntime string `toml:"default_runtime"`
@@ -258,10 +273,17 @@ type RuntimeConfig struct {
 	// default for the runtime.
 	ApparmorProfile string `toml:"apparmor_profile"`
 
+	// BlockIOConfigFile is the path to the blockio class configuration
+	// file for configuring the cgroup blockio controller.
+	BlockIOConfigFile string `toml:"blockio_config_file"`
+
 	// IrqBalanceConfigFile is the irqbalance service config file which is used
 	// for configuring irqbalance daemon.
 	IrqBalanceConfigFile string `toml:"irqbalance_config_file"`
 
+	// RdtConfigFile is the RDT config file used for configuring resctrl fs
+	RdtConfigFile string `toml:"rdt_config_file"`
+
 	// CgroupManagerName is the manager implementation name which is used to
 	// handle cgroups for containers.
 	CgroupManagerName string `toml:"cgroup_manager"`
@@ -345,6 +367,12 @@ type RuntimeConfig struct {
 	// apparmorConfig is the internal AppArmor configuration
 	apparmorConfig *apparmor.Config
 
+	// blockioConfig is the internal blockio configuration
+	blockioConfig *blockio.Config
+
+	// rdtConfig is the internal Rdt configuration
+	rdtConfig *rdt.Config
+
 	// ulimitConfig is the internal ulimit configuration
 	ulimitsConfig *ulimits.Config
 
@@ -457,11 +485,20 @@ type MetricsConfig struct {
 	// EnableMetrics can be used to globally enable or disable metrics support
 	EnableMetrics bool `toml:"enable_metrics"`
 
+	// MetricsCollectors specifies enabled metrics collectors.
+	MetricsCollectors collectors.Collectors `toml:"metrics_collectors"`
+
 	// MetricsPort is the port on which the metrics server will listen.
 	MetricsPort int `toml:"metrics_port"`
 
 	// Local socket path to bind the metrics server to
 	MetricsSocket string `toml:"metrics_socket"`
+
+	// MetricsCert is the certificate for the secure metrics server.
+	MetricsCert string `toml:"metrics_cert"`
+
+	// MetricsKey is the certificate key for the secure metrics server.
+	MetricsKey string `toml:"metrics_key"`
 }
 
 // tomlConfig is another way of looking at a Config, which is
@@ -629,7 +666,7 @@ func (c *Config) ToBytes() ([]byte, error) {
 
 // DefaultConfig returns the default configuration for crio.
 func DefaultConfig() (*Config, error) {
-	storeOpts, err := storage.DefaultStoreOptions(false, rootless.GetRootlessUID())
+	storeOpts, err := storage.DefaultStoreOptions(rootless.IsRootless(), rootless.GetRootlessUID())
 	if err != nil {
 		return nil, err
 	}
@@ -647,6 +684,7 @@ func DefaultConfig() (*Config, error) {
 			VersionFile:        CrioVersionPathTmp,
 			VersionFilePersist: CrioVersionPathPersist,
 			CleanShutdownFile:  CrioCleanShutdownFile,
+			InternalWipe:       true,
 		},
 		APIConfig: APIConfig{
 			Listen:             CrioSocketPath,
@@ -673,7 +711,9 @@ func DefaultConfig() (*Config, error) {
 			ConmonCgroup:             "system.slice",
 			SELinux:                  selinuxEnabled(),
 			ApparmorProfile:          apparmor.DefaultProfile,
+			BlockIOConfigFile:        DefaultBlockIOConfigFile,
 			IrqBalanceConfigFile:     DefaultIrqBalanceConfigFile,
+			RdtConfigFile:            rdt.DefaultRdtConfigFile,
 			CgroupManagerName:        cgroupManager.Name(),
 			PidsLimit:                DefaultPidsLimit,
 			ContainerExitsDir:        containerExitsDir,
@@ -684,8 +724,11 @@ func DefaultConfig() (*Config, error) {
 			LogLevel:                 "info",
 			HooksDir:                 []string{hooks.DefaultDir},
 			NamespacesDir:            defaultNamespacesDir,
+			DropInfraCtr:             true,
 			seccompConfig:            seccomp.New(),
 			apparmorConfig:           apparmor.New(),
+			blockioConfig:            blockio.New(),
+			rdtConfig:                rdt.New(),
 			ulimitsConfig:            ulimits.New(),
 			cgroupManager:            cgroupManager,
 			deviceConfig:             device.New(),
@@ -702,7 +745,8 @@ func DefaultConfig() (*Config, error) {
 			PluginDirs: []string{cniBinDir},
 		},
 		MetricsConfig: MetricsConfig{
-			MetricsPort: 9090,
+			MetricsPort:       9090,
+			MetricsCollectors: collectors.All(),
 		},
 	}, nil
 }
@@ -844,9 +888,10 @@ func (c *RuntimeConfig) Validate(systemContext *types.SystemContext, onExecution
 			// first. If it does not exist then we add runc + its path to the runtimes map.
 			if _, ok := c.Runtimes[defaultRuntime]; !ok {
 				c.Runtimes[defaultRuntime] = &RuntimeHandler{
-					RuntimePath: "",
-					RuntimeType: DefaultRuntimeType,
-					RuntimeRoot: DefaultRuntimeRoot,
+					RuntimePath:       "",
+					RuntimeType:       DefaultRuntimeType,
+					RuntimeRoot:       DefaultRuntimeRoot,
+					RuntimeConfigPath: "",
 				}
 			}
 			// Set the DefaultRuntime to runc so we don't fail further along in the code
@@ -869,7 +914,7 @@ func (c *RuntimeConfig) Validate(systemContext *types.SystemContext, onExecution
 	// value defined in the configuration file to be too low.
 	if c.CtrStopTimeout < defaultCtrStopTimeout {
 		c.CtrStopTimeout = defaultCtrStopTimeout
-		logrus.Warnf("forcing ctr_stop_timeout to lowest possible value of %ds", c.CtrStopTimeout)
+		logrus.Warnf("Forcing ctr_stop_timeout to lowest possible value of %ds", c.CtrStopTimeout)
 	}
 
 	if _, err := c.Sysctls(); err != nil {
@@ -916,15 +961,15 @@ func (c *RuntimeConfig) Validate(systemContext *types.SystemContext, onExecution
 		for _, hooksDir := range c.HooksDir {
 			if err := utils.IsDirectory(hooksDir); err != nil {
 				if !os.IsNotExist(err) {
-					logrus.Warnf("skipping invalid hooks directory: %s exists but is not a directory", hooksDir)
+					logrus.Warnf("Skipping invalid hooks directory: %s exists but is not a directory", hooksDir)
 					continue
 				}
 				if err := os.MkdirAll(hooksDir, 0o755); err != nil {
-					logrus.Debugf("failed to create requested hooks dir: %v", err)
+					logrus.Debugf("Failed to create requested hooks dir: %v", err)
 					continue
 				}
 			}
-			logrus.Debugf("using hooks directory: %s", hooksDir)
+			logrus.Debugf("Using hooks directory: %s", hooksDir)
 			hooksDirs = append(hooksDirs, hooksDir)
 			continue
 		}
@@ -956,6 +1001,15 @@ func (c *RuntimeConfig) Validate(systemContext *types.SystemContext, onExecution
 		if err := c.apparmorConfig.LoadProfile(c.ApparmorProfile); err != nil {
 			return errors.Wrap(err, "unable to load AppArmor profile")
 		}
+
+		if err := c.blockioConfig.Load(c.BlockIOConfigFile); err != nil {
+			return errors.Wrap(err, "blockio configuration")
+		}
+
+		if err := c.rdtConfig.Load(c.RdtConfigFile); err != nil {
+			return errors.Wrap(err, "rdt configuration")
+		}
+
 		cgroupManager, err := cgmgr.SetCgroupManager(c.CgroupManagerName)
 		if err != nil {
 			return errors.Wrap(err, "unable to update cgroup manager")
@@ -1011,6 +1065,10 @@ func (c *RuntimeConfig) ConmonSupportsSync() bool {
 	return c.conmonManager.SupportsSync()
 }
 
+func (c *RuntimeConfig) ConmonSupportsLogGlobalSizeMax() bool {
+	return c.conmonManager.SupportsLogGlobalSizeMax()
+}
+
 func (c *RuntimeConfig) ValidatePinnsPath(executable string) error {
 	var err error
 	c.PinnsPath, err = validateExecutablePath(executable, c.PinnsPath)
@@ -1028,6 +1086,16 @@ func (c *RuntimeConfig) AppArmor() *apparmor.Config {
 	return c.apparmorConfig
 }
 
+// BlockIO returns the blockio configuration
+func (c *RuntimeConfig) BlockIO() *blockio.Config {
+	return c.blockioConfig
+}
+
+// Rdt returns the RDT configuration
+func (c *RuntimeConfig) Rdt() *rdt.Config {
+	return c.rdtConfig
+}
+
 // CgroupManager returns the CgroupManager configuration
 func (c *RuntimeConfig) CgroupManager() cgmgr.CgroupManager {
 	return c.cgroupManager
@@ -1053,13 +1121,13 @@ func validateExecutablePath(executable, currentPath string) (string, error) {
 		if err != nil {
 			return "", err
 		}
-		logrus.Debugf("using %s from $PATH: %s", executable, path)
+		logrus.Debugf("Using %s from $PATH: %s", executable, path)
 		return path, nil
 	}
 	if _, err := os.Stat(currentPath); err != nil {
 		return "", errors.Wrapf(err, "invalid %s path", executable)
 	}
-	logrus.Infof("using %s executable: %s", executable, currentPath)
+	logrus.Infof("Using %s executable: %s", executable, currentPath)
 	return currentPath, nil
 }
 
@@ -1118,6 +1186,9 @@ func (r *RuntimeHandler) Validate(name string) error {
 	if err := r.ValidateRuntimePath(name); err != nil {
 		return err
 	}
+	if err := r.ValidateRuntimeConfigPath(name); err != nil {
+		return err
+	}
 	if err := r.ValidateRuntimeAllowedAnnotations(); err != nil {
 		return err
 	}
@@ -1149,7 +1220,7 @@ func (r *RuntimeHandler) ValidateRuntimePath(name string) error {
 			return errors.Wrapf(err, "%q not found in $PATH", name)
 		}
 		r.RuntimePath = executable
-		logrus.Debugf("using runtime executable from $PATH %q", executable)
+		logrus.Debugf("Using runtime executable from $PATH %q", executable)
 	} else if _, err := os.Stat(r.RuntimePath); err != nil && os.IsNotExist(err) {
 		return fmt.Errorf("invalid runtime_path for runtime '%s': %q",
 			name, err)
@@ -1176,6 +1247,21 @@ func (r *RuntimeHandler) ValidateRuntimeType(name string) error {
 	return nil
 }
 
+// ValidateRuntimeConfigPath checks if the `RuntimeConfigPath` exists.
+func (r *RuntimeHandler) ValidateRuntimeConfigPath(name string) error {
+	if r.RuntimeConfigPath == "" {
+		return nil
+	}
+	if r.RuntimeType != RuntimeTypeVM {
+		return fmt.Errorf("runtime_config_path can only be used with the 'vm' runtime type")
+	}
+	if _, err := os.Stat(r.RuntimeConfigPath); err != nil && os.IsNotExist(err) {
+		return fmt.Errorf("invalid runtime_config_path for runtime '%s': %q",
+			name, err)
+	}
+	return nil
+}
+
 func (r *RuntimeHandler) ValidateRuntimeAllowedAnnotations() error {
 	disallowedAnnotations := make(map[string]struct{})
 	for _, ann := range annotations.AllAllowedAnnotations {
@@ -1207,10 +1293,15 @@ func (c *NetworkConfig) CNIPluginReadyOrError() error {
 }
 
 // CNIPluginAddWatcher returns the network configuration CNI plugin
-func (c *NetworkConfig) CNIPluginAddWatcher() chan struct{} {
+func (c *NetworkConfig) CNIPluginAddWatcher() chan bool {
 	return c.cniManager.AddWatcher()
 }
 
+// CNIManagerShutdown shuts down the CNI Manager
+func (c *NetworkConfig) CNIManagerShutdown() {
+	c.cniManager.Shutdown()
+}
+
 // SetSingleConfigPath set single config path for config
 func (c *Config) SetSingleConfigPath(singleConfigPath string) {
 	c.singleConfigPath = singleConfigPath
