@@ -4,7 +4,10 @@ import (
 	"io/ioutil"
 	"os"
 	"path"
+	"path/filepath"
 
+	"github.com/containers/storage"
+	crioann "github.com/cri-o/cri-o/pkg/annotations"
 	"github.com/cri-o/cri-o/pkg/config"
 
 	. "github.com/onsi/ginkgo"
@@ -15,35 +18,47 @@ import (
 var _ = t.Describe("Config", func() {
 	BeforeEach(beforeEach)
 
-	var runtimeValidConfig = func() *config.Config {
+	runtimeValidConfig := func() *config.Config {
 		sut.Runtimes["runc"] = &config.RuntimeHandler{
 			RuntimePath: validFilePath, RuntimeType: config.DefaultRuntimeType,
 		}
-		sut.Conmon = validFilePath
+		sut.PinnsPath = validFilePath
+		sut.NamespacesDir = os.TempDir()
+		sut.Conmon = validConmonPath()
 		tmpDir := t.MustTempDir("cni-test")
 		sut.NetworkConfig.PluginDirs = []string{tmpDir}
 		sut.NetworkDir = os.TempDir()
 		sut.LogDir = "/"
 		sut.Listen = t.MustTempFile("crio.sock")
+		sut.HooksDir = []string{}
 		return sut
 	}
 
+	isRootless := func() bool {
+		return os.Geteuid() != 0
+	}
+
 	t.Describe("ValidateConfig", func() {
 		It("should succeed with default config", func() {
 			// Given
+
 			// When
-			err := sut.Validate(nil, false)
+			err := sut.Validate(false)
 
 			// Then
 			Expect(err).To(BeNil())
 		})
 
 		It("should succeed with runtime checks", func() {
+			if isRootless() {
+				Skip("this test does not work rootless")
+			}
+
 			// Given
 			sut = runtimeValidConfig()
 
 			// When
-			err := sut.Validate(nil, true)
+			err := sut.Validate(true)
 
 			// Then
 			Expect(err).To(BeNil())
@@ -54,7 +69,7 @@ var _ = t.Describe("Config", func() {
 			sut.RootConfig.LogDir = "/dev/null"
 
 			// When
-			err := sut.Validate(nil, true)
+			err := sut.Validate(true)
 
 			// Then
 			Expect(err).NotTo(BeNil())
@@ -66,7 +81,7 @@ var _ = t.Describe("Config", func() {
 			sut.Listen = "/proc"
 
 			// When
-			err := sut.Validate(nil, true)
+			err := sut.Validate(true)
 
 			// Then
 			Expect(err).NotTo(BeNil())
@@ -77,7 +92,7 @@ var _ = t.Describe("Config", func() {
 			sut.AdditionalDevices = []string{invalidPath}
 
 			// When
-			err := sut.Validate(nil, true)
+			err := sut.Validate(true)
 
 			// Then
 			Expect(err).NotTo(BeNil())
@@ -86,11 +101,11 @@ var _ = t.Describe("Config", func() {
 		It("should fail with invalid network config", func() {
 			// Given
 			sut.Runtimes["runc"] = &config.RuntimeHandler{RuntimePath: validDirPath}
-			sut.Conmon = validFilePath
+			sut.Conmon = validConmonPath()
 			sut.NetworkConfig.NetworkDir = invalidPath
 
 			// When
-			err := sut.Validate(nil, true)
+			err := sut.Validate(true)
 
 			// Then
 			Expect(err).NotTo(BeNil())
@@ -101,7 +116,7 @@ var _ = t.Describe("Config", func() {
 			sut.ImageVolumes = invalidPath
 
 			// When
-			err := sut.Validate(nil, false)
+			err := sut.Validate(false)
 
 			// Then
 			Expect(err).NotTo(BeNil())
@@ -112,12 +127,11 @@ var _ = t.Describe("Config", func() {
 			sut.DefaultUlimits = []string{"invalid=-1:-1"}
 
 			// When
-			err := sut.Validate(nil, false)
+			err := sut.Validate(false)
 
 			// Then
 			Expect(err).NotTo(BeNil())
 		})
-
 	})
 
 	t.Describe("ValidateAPIConfig", func() {
@@ -180,11 +194,7 @@ var _ = t.Describe("Config", func() {
 
 		It("should succeed during runtime", func() {
 			// Given
-			sut.Runtimes["runc"] = &config.RuntimeHandler{
-				RuntimePath: validFilePath,
-				RuntimeType: config.DefaultRuntimeType,
-			}
-			sut.Conmon = validFilePath
+			sut = runtimeValidConfig()
 
 			// When
 			err := sut.RuntimeConfig.Validate(nil, true)
@@ -195,12 +205,8 @@ var _ = t.Describe("Config", func() {
 
 		It("should succeed with additional devices", func() {
 			// Given
+			sut = runtimeValidConfig()
 			sut.AdditionalDevices = []string{"/dev/null:/dev/null:rw"}
-			sut.Runtimes["runc"] = &config.RuntimeHandler{
-				RuntimePath: validFilePath,
-				RuntimeType: config.DefaultRuntimeType,
-			}
-			sut.Conmon = validFilePath
 
 			// When
 			err := sut.RuntimeConfig.Validate(nil, true)
@@ -215,40 +221,49 @@ var _ = t.Describe("Config", func() {
 				RuntimePath: validFilePath,
 				RuntimeType: config.DefaultRuntimeType,
 			}
-			sut.Conmon = validFilePath
-			sut.HooksDir = []string{validDirPath}
+			sut.PinnsPath = validFilePath
+			sut.NamespacesDir = os.TempDir()
+			sut.Conmon = validConmonPath()
+			sut.HooksDir = []string{validDirPath, validDirPath, validDirPath}
 
 			// When
 			err := sut.RuntimeConfig.Validate(nil, true)
 
 			// Then
 			Expect(err).To(BeNil())
+			Expect(sut.HooksDir).To(HaveLen(3))
 		})
 
-		It("should fail on invalid hooks directory", func() {
+		It("should sort out invalid hooks directories", func() {
 			// Given
 			sut.Runtimes["runc"] = &config.RuntimeHandler{RuntimePath: validFilePath}
-			sut.Conmon = validFilePath
-			sut.HooksDir = []string{invalidPath}
+			sut.Conmon = validConmonPath()
+			sut.PinnsPath = validFilePath
+			sut.NamespacesDir = os.TempDir()
+			sut.HooksDir = []string{invalidPath, validDirPath, validDirPath}
 
 			// When
 			err := sut.RuntimeConfig.Validate(nil, true)
 
 			// Then
-			Expect(err).NotTo(BeNil())
+			Expect(err).To(BeNil())
+			Expect(sut.HooksDir).To(HaveLen(2))
 		})
 
-		It("should fail if the hooks directory is not a directory", func() {
+		It("should create non-existent hooks directory", func() {
 			// Given
 			sut.Runtimes["runc"] = &config.RuntimeHandler{RuntimePath: validFilePath}
-			sut.Conmon = validFilePath
-			sut.HooksDir = []string{validFilePath}
+			sut.Conmon = validConmonPath()
+			sut.PinnsPath = validFilePath
+			sut.NamespacesDir = os.TempDir()
+			sut.HooksDir = []string{filepath.Join(validDirPath, "new")}
 
 			// When
 			err := sut.RuntimeConfig.Validate(nil, true)
 
 			// Then
-			Expect(err).NotTo(BeNil())
+			Expect(err).To(BeNil())
+			Expect(sut.HooksDir).To(HaveLen(1))
 		})
 
 		It("should fail on invalid conmon path", func() {
@@ -352,36 +367,12 @@ var _ = t.Describe("Config", func() {
 			Expect(err).NotTo(BeNil())
 		})
 
-		It("should fail wrong UID mappings", func() {
-			// Given
-			sut.UIDMappings = "value"
-			sut.ManageNetworkNSLifecycle = true
-
-			// When
-			err := sut.Validate(nil, false)
-
-			// Then
-			Expect(err).NotTo(BeNil())
-		})
-
-		It("should fail wrong GID mappings", func() {
-			// Given
-			sut.GIDMappings = "value"
-			sut.ManageNetworkNSLifecycle = true
-
-			// When
-			err := sut.Validate(nil, false)
-
-			// Then
-			Expect(err).NotTo(BeNil())
-		})
-
 		It("should fail wrong max log size", func() {
 			// Given
 			sut.LogSizeMax = 1
 
 			// When
-			err := sut.Validate(nil, false)
+			err := sut.Validate(false)
 
 			// Then
 			Expect(err).NotTo(BeNil())
@@ -422,6 +413,28 @@ var _ = t.Describe("Config", func() {
 			Expect(err).To(BeNil())
 			Expect(sut.DefaultRuntime).To(Equal("runc"))
 		})
+
+		It("should fail on invalid default_sysctls", func() {
+			// Given
+			sut.DefaultSysctls = []string{"invalid"}
+
+			// When
+			err := sut.RuntimeConfig.Validate(nil, false)
+
+			// Then
+			Expect(err).NotTo(BeNil())
+		})
+
+		It("should fail on invalid conmon cgroup", func() {
+			// Given
+			sut.ConmonCgroup = "invalid"
+
+			// When
+			err := sut.RuntimeConfig.Validate(nil, false)
+
+			// Then
+			Expect(err).NotTo(BeNil())
+		})
 	})
 
 	t.Describe("ValidateRuntimes", func() {
@@ -447,9 +460,10 @@ var _ = t.Describe("Config", func() {
 			Expect(err).To(BeNil())
 		})
 
-		It("should fail if executable not in $PATH", func() {
+		It("should fail if default executable not in $PATH", func() {
 			// Given
 			sut.Runtimes[invalidPath] = &config.RuntimeHandler{RuntimePath: ""}
+			sut.DefaultRuntime = invalidPath
 
 			// When
 			err := sut.RuntimeConfig.ValidateRuntimes()
@@ -458,6 +472,18 @@ var _ = t.Describe("Config", func() {
 			Expect(err).NotTo(BeNil())
 		})
 
+		It("should not fail if non-default executable not in $PATH", func() {
+			// Given
+			sut.Runtimes[invalidPath] = &config.RuntimeHandler{RuntimePath: ""}
+			sut.DefaultRuntime = "runc"
+
+			// When
+			err := sut.RuntimeConfig.ValidateRuntimes()
+
+			// Then
+			Expect(err).To(BeNil())
+		})
+
 		It("should fail with wrong but set runtime_path", func() {
 			// Given
 			sut.Runtimes["runc"] = &config.RuntimeHandler{RuntimePath: invalidPath}
@@ -482,6 +508,35 @@ var _ = t.Describe("Config", func() {
 			// Then
 			Expect(err).NotTo(BeNil())
 		})
+
+		It("should fail with wrong allowed_annotation", func() {
+			// Given
+			sut.Runtimes["runc"] = &config.RuntimeHandler{
+				RuntimePath:        validFilePath,
+				AllowedAnnotations: []string{"wrong"},
+			}
+
+			// When
+			err := sut.RuntimeConfig.ValidateRuntimes()
+
+			// Then
+			Expect(err).NotTo(BeNil())
+		})
+		It("should have allowed and disallowed annotation", func() {
+			// Given
+			sut.Runtimes["runc"] = &config.RuntimeHandler{
+				RuntimePath:        validFilePath,
+				AllowedAnnotations: []string{crioann.DevicesAnnotation},
+			}
+
+			// When
+			err := sut.RuntimeConfig.ValidateRuntimes()
+
+			// Then
+			Expect(err).To(BeNil())
+			Expect(sut.Runtimes["runc"].AllowedAnnotations).To(ContainElement(crioann.DevicesAnnotation))
+			Expect(sut.Runtimes["runc"].DisallowedAnnotations).NotTo(ContainElement(crioann.DevicesAnnotation))
+		})
 	})
 
 	t.Describe("ValidateConmonPath", func() {
@@ -490,11 +545,11 @@ var _ = t.Describe("Config", func() {
 			sut.RuntimeConfig.Conmon = ""
 
 			// When
-			err := sut.RuntimeConfig.ValidateConmonPath(validFilePath)
+			err := sut.RuntimeConfig.ValidateConmonPath(validConmonPath())
 
 			// Then
 			Expect(err).To(BeNil())
-			Expect(sut.RuntimeConfig.Conmon).To(Equal(validFilePath))
+			Expect(sut.RuntimeConfig.Conmon).To(Equal(validConmonPath()))
 		})
 
 		It("should fail with invalid file in $PATH", func() {
@@ -510,7 +565,7 @@ var _ = t.Describe("Config", func() {
 
 		It("should succeed with valid file outside $PATH", func() {
 			// Given
-			sut.RuntimeConfig.Conmon = validDirPath
+			sut.RuntimeConfig.Conmon = validConmonPath()
 
 			// When
 			err := sut.RuntimeConfig.ValidateConmonPath("")
@@ -543,10 +598,7 @@ var _ = t.Describe("Config", func() {
 
 		It("should succeed during runtime", func() {
 			// Given
-			sut.NetworkConfig.NetworkDir = validDirPath
-			tmpDir := path.Join(os.TempDir(), "cni-test")
-			sut.NetworkConfig.PluginDirs = []string{tmpDir}
-			defer os.RemoveAll(tmpDir)
+			sut = runtimeValidConfig()
 
 			// When
 			err := sut.NetworkConfig.Validate(true)
@@ -650,6 +702,10 @@ var _ = t.Describe("Config", func() {
 		})
 
 		It("should succeed during runtime", func() {
+			if isRootless() {
+				Skip("this test does not work rootless")
+			}
+
 			// Given
 			sut = runtimeValidConfig()
 
@@ -676,11 +732,57 @@ var _ = t.Describe("Config", func() {
 			sut.RootConfig.LogDir = "test"
 
 			// When
-			err := sut.Validate(nil, true)
+			err := sut.Validate(true)
 
 			// Then
 			Expect(err).NotTo(BeNil())
 		})
+
+		It("should get default storage options when options are empty", func() {
+			if isRootless() {
+				Skip("this test does not work rootless")
+			}
+
+			// Given
+			defaultStore, err := storage.GetStore(storage.StoreOptions{})
+			Expect(err).To(BeNil())
+
+			sut.RootConfig.RunRoot = ""
+			sut.RootConfig.Root = ""
+			sut.RootConfig.Storage = ""
+			sut.RootConfig.StorageOptions = make([]string, 0)
+
+			// When
+			err = sut.Validate(true)
+
+			// Then
+			Expect(err).To(BeNil())
+			Expect(sut.RootConfig.RunRoot).To(Equal(defaultStore.RunRoot()))
+			Expect(sut.RootConfig.Root).To(Equal(defaultStore.GraphRoot()))
+			Expect(sut.RootConfig.Storage).To(Equal(defaultStore.GraphDriverName()))
+			Expect(sut.RootConfig.StorageOptions).To(Equal(defaultStore.GraphOptions()))
+		})
+
+		It("should override default storage options", func() {
+			if isRootless() {
+				Skip("this test does not work rootless")
+			}
+
+			// Given
+			defaultStore, err := storage.GetStore(storage.StoreOptions{})
+			Expect(err).To(BeNil())
+
+			sut.RootConfig.RunRoot = "/tmp"
+			sut.RootConfig.Root = "/tmp"
+
+			// When
+			err = sut.Validate(true)
+
+			// Then
+			Expect(err).To(BeNil())
+			Expect(sut.RootConfig.RunRoot).NotTo(Equal(defaultStore.RunRoot()))
+			Expect(sut.RootConfig.Root).NotTo(Equal(defaultStore.GraphRoot()))
+		})
 	})
 
 	t.Describe("ToFile", func() {
@@ -702,7 +804,7 @@ var _ = t.Describe("Config", func() {
 		It("should fail with invalid path", func() {
 			// Given
 			// When
-			err := sut.ToFile("/proc/invalid")
+			err := sut.ToFile(invalidPath)
 
 			// Then
 			Expect(err).NotTo(BeNil())
@@ -710,17 +812,96 @@ var _ = t.Describe("Config", func() {
 	})
 
 	t.Describe("UpdateFromFile", func() {
-		It("should succeed with default config", func() {
+		It("should succeed with custom config", func() {
 			// Given
+			f := t.MustTempFile("config")
+			Expect(ioutil.WriteFile(f,
+				[]byte(`
+					[crio]
+					storage_driver = "overlay2"
+					[crio.runtime]
+					pids_limit = 2048`,
+				), 0),
+			).To(BeNil())
+
 			// When
-			err := sut.UpdateFromFile("testdata/config.toml")
+			err := sut.UpdateFromFile(f)
 
 			// Then
 			Expect(err).To(BeNil())
 			Expect(sut.Storage).To(Equal("overlay2"))
+			Expect(sut.Runtimes).To(HaveLen(1))
+			Expect(sut.Runtimes).To(HaveKey("runc"))
 			Expect(sut.PidsLimit).To(BeEquivalentTo(2048))
 		})
 
+		It("should inherit storage_options from storage.conf and remove duplicates", func() {
+			f := t.MustTempFile("config")
+			// Given
+			Expect(ioutil.WriteFile(f,
+				[]byte(`
+					[crio]
+					storage_option = [
+						"foo=bar",
+					]`,
+				), 0),
+			).To(BeNil())
+			for _, tc := range []struct {
+				opts   []string
+				expect []string
+			}{
+				{[]string{"option1=v1", "option2=v2", "option3=v3"}, []string{"option1=v1", "option2=v2", "option3=v3", "foo=bar"}},
+				{[]string{"option1=v1", "option3=v3", "option2=v2", "option3=v3"}, []string{"option1=v1", "option2=v2", "option3=v3", "foo=bar"}},
+				{[]string{"option1=v1", "option2=v2", "option3=v3", "option1=v1"}, []string{"option2=v2", "option3=v3", "option1=v1", "foo=bar"}},
+				{[]string{"option1=v1", "option2=v2", "option3=v3", "option4=v4", "option3=v3", "option1=v1"}, []string{"option2=v2", "option4=v4", "option3=v3", "option1=v1", "foo=bar"}},
+			} {
+				// When
+				defaultcfg := defaultConfig()
+				defaultcfg.StorageOptions = tc.opts
+				err := defaultcfg.UpdateFromFile(f)
+
+				// Then
+				Expect(err).To(BeNil())
+				Expect(defaultcfg.RootConfig.StorageOptions).To(Equal(tc.expect))
+			}
+		})
+
+		It("should succeed with custom runtime", func() {
+			// Given
+			f := t.MustTempFile("config")
+			Expect(ioutil.WriteFile(f,
+				[]byte("[crio.runtime.runtimes.crun]"), 0),
+			).To(BeNil())
+
+			// When
+			err := sut.UpdateFromFile(f)
+
+			// Then
+			Expect(err).To(BeNil())
+			Expect(sut.Runtimes).To(HaveLen(2))
+			Expect(sut.Runtimes).To(HaveKey("crun"))
+		})
+
+		It("should succeed with additional runtime", func() {
+			// Given
+			f := t.MustTempFile("config")
+			Expect(ioutil.WriteFile(f,
+				[]byte(`
+					[crio.runtime.runtimes.runc]
+					[crio.runtime.runtimes.crun]
+				`), 0),
+			).To(BeNil())
+
+			// When
+			err := sut.UpdateFromFile(f)
+
+			// Then
+			Expect(err).To(BeNil())
+			Expect(sut.Runtimes).To(HaveLen(2))
+			Expect(sut.Runtimes).To(HaveKey("crun"))
+			Expect(sut.Runtimes).To(HaveKey("runc"))
+		})
+
 		It("should fail when file does not exist", func() {
 			// Given
 			// When
@@ -786,4 +967,124 @@ var _ = t.Describe("Config", func() {
 			Expect(res).NotTo(BeNil())
 		})
 	})
+
+	t.Describe("UpdateFromPath", func() {
+		It("should succeed with the correct priority", func() {
+			// Given
+			Expect(sut.LogLevel).To(Equal("info"))
+
+			configDir := t.MustTempDir("config-dir")
+			Expect(ioutil.WriteFile(
+				filepath.Join(configDir, "00-default"),
+				[]byte("[crio.runtime]\nlog_level = \"debug\"\n"),
+				0o644,
+			)).To(BeNil())
+			Expect(ioutil.WriteFile(
+				filepath.Join(configDir, "01-my-config"),
+				[]byte("[crio.runtime]\nlog_level = \"warning\"\n"),
+				0o644,
+			)).To(BeNil())
+
+			// When
+			err := sut.UpdateFromPath(configDir)
+
+			// Then
+			Expect(err).To(BeNil())
+			Expect(sut.LogLevel).To(Equal("warning"))
+		})
+
+		It("should fail with invalid config", func() {
+			// Given
+			configDir := t.MustTempDir("config-dir")
+			Expect(ioutil.WriteFile(
+				filepath.Join(configDir, "00-default"),
+				[]byte("[crio.runtime]\nlog_level = true\n"),
+				0o644,
+			)).To(BeNil())
+
+			// When
+			err := sut.UpdateFromPath(configDir)
+
+			// Then
+			Expect(err).NotTo(BeNil())
+		})
+
+		It("should succeed with not existing path", func() {
+			// Given
+			// When
+			err := sut.UpdateFromPath("not-existing")
+
+			// Then
+			Expect(err).To(BeNil())
+		})
+	})
+
+	t.Describe("ValidateRuntimeVMBinaryPattern", func() {
+		It("should succeed when using RuntimeTypeVM and runtime_path follows the containerd pattern", func() {
+			// Given
+			sut.Runtimes["kata"] = &config.RuntimeHandler{
+				RuntimePath: "containerd-shim-kata-qemu-v2", RuntimeType: config.RuntimeTypeVM,
+			}
+
+			// When
+			ok := sut.Runtimes["kata"].ValidateRuntimeVMBinaryPattern()
+
+			// Then
+			Expect(ok).To(BeTrue())
+		})
+
+		It("should fail when using RuntimeTypeVM and runtime_path does not follow the containerd pattern", func() {
+			// Given
+			sut.Runtimes["kata"] = &config.RuntimeHandler{
+				RuntimePath: "kata-runtime", RuntimeType: config.RuntimeTypeVM,
+			}
+
+			// When
+			ok := sut.Runtimes["kata"].ValidateRuntimeVMBinaryPattern()
+
+			// Then
+			Expect(ok).To(BeFalse())
+		})
+	})
+
+	t.Describe("ValidateRuntimeConfigPath", func() {
+		It("should fail with OCI runtime type when runtime_config_path is used", func() {
+			// Given
+			sut.Runtimes["runc"] = &config.RuntimeHandler{
+				RuntimeConfigPath: validFilePath, RuntimeType: config.DefaultRuntimeType,
+			}
+
+			// When
+			err := sut.Runtimes["runc"].ValidateRuntimeConfigPath("runc")
+
+			// Then
+			Expect(err).NotTo(BeNil())
+		})
+
+		It("should fail with VM runtime type and runtime_config_path points to an invalid path", func() {
+			// Given
+			sut.Runtimes["kata"] = &config.RuntimeHandler{
+				RuntimeConfigPath: invalidPath, RuntimeType: config.RuntimeTypeVM,
+			}
+
+			// When
+			err := sut.Runtimes["kata"].ValidateRuntimeConfigPath("kata")
+
+			// Then
+			Expect(err).NotTo(BeNil())
+		})
+
+		It("should succeed with VM runtime type and runtime_config_path points to a valid path", func() {
+			// Given
+			sut.Runtimes["kata"] = &config.RuntimeHandler{
+				RuntimeConfigPath: validFilePath, RuntimeType: config.RuntimeTypeVM,
+			}
+
+			// When
+			err := sut.Runtimes["kata"].ValidateRuntimeConfigPath("kata")
+
+			// Then
+			Expect(err).To(BeNil())
+		})
+	})
 })
