@@ -93,7 +93,7 @@ type Container interface {
 	SpecAddAnnotations(ctx context.Context, sandbox *sandbox.Sandbox, containerVolume []oci.ContainerVolume, mountPoint, configStopSignal string, imageResult *storage.ImageResult, isSystemd, systemdHasCollectMode bool) error
 
 	// SpecAddDevices adds devices from the server config, and container CRI config
-	SpecAddDevices([]device.Device, []device.Device, bool) error
+	SpecAddDevices([]device.Device, []device.Device, bool, bool) error
 
 	// AddUnifiedResourcesFromAnnotations adds the cgroup-v2 resources specified in the io.kubernetes.cri-o.UnifiedCgroup annotation
 	AddUnifiedResourcesFromAnnotations(annotationsMap map[string]string) error
@@ -402,7 +402,7 @@ func (c *container) LogPath(sboxLogDir string) (string, error) {
 		return "", err
 	}
 
-	logrus.Debugf("setting container's log_path = %s, sbox.logdir = %s, ctr.logfile = %s",
+	logrus.Debugf("Setting container's log_path = %s, sbox.logdir = %s, ctr.logfile = %s",
 		sboxLogDir, c.config.LogPath, logPath,
 	)
 	return logPath, nil
