@@ -1,3 +1,4 @@
+//go:build linux
 // +build linux
 
 package server
@@ -11,32 +12,41 @@ import (
 	"syscall"
 	"time"
 
-	"github.com/containers/buildah/pkg/secrets"
 	"github.com/containers/buildah/util"
-	"github.com/containers/libpod/v2/pkg/rootless"
-	selinux "github.com/containers/libpod/v2/pkg/selinux"
+	"github.com/containers/common/pkg/subscriptions"
+	"github.com/containers/podman/v3/pkg/rootless"
+	selinux "github.com/containers/podman/v3/pkg/selinux"
 	cstorage "github.com/containers/storage"
 	"github.com/containers/storage/pkg/idtools"
 	"github.com/containers/storage/pkg/mount"
 	"github.com/cri-o/cri-o/internal/config/cgmgr"
 	"github.com/cri-o/cri-o/internal/config/device"
 	"github.com/cri-o/cri-o/internal/config/node"
+	"github.com/cri-o/cri-o/internal/config/rdt"
 	"github.com/cri-o/cri-o/internal/lib/sandbox"
 	"github.com/cri-o/cri-o/internal/log"
 	oci "github.com/cri-o/cri-o/internal/oci"
 	"github.com/cri-o/cri-o/internal/storage"
 	crioann "github.com/cri-o/cri-o/pkg/annotations"
 	ctrIface "github.com/cri-o/cri-o/pkg/container"
+	"github.com/cri-o/cri-o/server/cri/types"
 	securejoin "github.com/cyphar/filepath-securejoin"
 	rspec "github.com/opencontainers/runtime-spec/specs-go"
 	"github.com/opencontainers/runtime-tools/generate"
 	"github.com/pkg/errors"
+	"github.com/sirupsen/logrus"
 	"golang.org/x/net/context"
-	pb "k8s.io/cri-api/pkg/apis/runtime/v1alpha2"
+
+	"github.com/intel/goresctrl/pkg/blockio"
+)
+
+const (
+	cgroupSysFsPath        = "/sys/fs/cgroup"
+	cgroupSysFsSystemdPath = "/sys/fs/cgroup/systemd"
 )
 
 // createContainerPlatform performs platform dependent intermediate steps before calling the container's oci.Runtime().CreateContainer()
-func (s *Server) createContainerPlatform(container *oci.Container, cgroupParent string, idMappings *idtools.IDMappings) error {
+func (s *Server) createContainerPlatform(ctx context.Context, container *oci.Container, cgroupParent string, idMappings *idtools.IDMappings) error {
 	if idMappings != nil && !container.Spoofed() {
 		rootPair := idMappings.RootPair()
 		for _, path := range []string{container.BundlePath(), container.MountPoint()} {
@@ -48,7 +58,7 @@ func (s *Server) createContainerPlatform(container *oci.Container, cgroupParent
 			return err
 		}
 	}
-	return s.Runtime().CreateContainer(container, cgroupParent)
+	return s.Runtime().CreateContainer(ctx, container, cgroupParent)
 }
 
 // makeAccessible changes the path permission and each parent directory to have --x--x--x
@@ -132,13 +142,21 @@ func (s *Server) createSandboxContainer(ctx context.Context, ctr ctrIface.Contai
 
 	// eventually, we'd like to access all of these variables through the interface themselves, and do most
 	// of the translation between CRI config -> oci/storage container in the container package
+
+	// TODO: eventually, this should be in the container package, but it's going through a lot of churn
+	// and SpecAddAnnotations is already being passed too many arguments
+	// Filter early so any use of the annotations don't use the wrong values
+	if err := s.Runtime().FilterDisallowedAnnotations(sb.RuntimeHandler(), ctr.Config().Annotations); err != nil {
+		return nil, err
+	}
+
 	containerID := ctr.ID()
 	containerName := ctr.Name()
 	containerConfig := ctr.Config()
 	if err := ctr.SetPrivileged(); err != nil {
 		return nil, err
 	}
-	securityContext := containerConfig.GetLinux().GetSecurityContext()
+	securityContext := containerConfig.Linux.SecurityContext
 
 	// creates a spec Generator with the default spec.
 	specgen := ctr.Spec()
@@ -163,7 +181,7 @@ func (s *Server) createSandboxContainer(ctx context.Context, ctr ctrIface.Contai
 			"/var/tmp": "mode=1777",
 		}
 		for target, mode := range mounts {
-			if !isInCRIMounts(target, containerConfig.GetMounts()) {
+			if !isInCRIMounts(target, containerConfig.Mounts) {
 				ctr.SpecAddMount(rspec.Mount{
 					Destination: target,
 					Type:        "tmpfs",
@@ -223,7 +241,7 @@ func (s *Server) createSandboxContainer(ctx context.Context, ctr ctrIface.Contai
 		idMappingOptions = &cstorage.IDMappingOptions{UIDMap: containerIDMappings.UIDs(), GIDMap: containerIDMappings.GIDs()}
 	}
 
-	metadata := containerConfig.GetMetadata()
+	metadata := containerConfig.Metadata
 
 	containerInfo, err := s.StorageRuntimeServer().CreateContainer(s.config.SystemContext,
 		sb.Name(), sb.ID(),
@@ -240,7 +258,7 @@ func (s *Server) createSandboxContainer(ctx context.Context, ctr ctrIface.Contai
 	}
 	defer func() {
 		if retErr != nil {
-			log.Infof(ctx, "createCtrLinux: deleting container %s from storage", containerInfo.ID)
+			log.Infof(ctx, "CreateCtrLinux: deleting container %s from storage", containerInfo.ID)
 			if err := s.StorageRuntimeServer().DeleteContainer(containerInfo.ID); err != nil {
 				log.Warnf(ctx, "Failed to cleanup container directory: %v", err)
 			}
@@ -252,9 +270,9 @@ func (s *Server) createSandboxContainer(ctx context.Context, ctr ctrIface.Contai
 	if !ctr.Privileged() {
 		processLabel = containerInfo.ProcessLabel
 	}
-	hostIPC := securityContext.GetNamespaceOptions().GetIpc() == pb.NamespaceMode_NODE
-	hostPID := securityContext.GetNamespaceOptions().GetPid() == pb.NamespaceMode_NODE
-	hostNet := securityContext.GetNamespaceOptions().GetNetwork() == pb.NamespaceMode_NODE
+	hostIPC := securityContext.NamespaceOptions.Ipc == types.NamespaceModeNODE
+	hostPID := securityContext.NamespaceOptions.Pid == types.NamespaceModeNODE
+	hostNet := securityContext.NamespaceOptions.Network == types.NamespaceModeNODE
 
 	// Don't use SELinux separation with Host Pid or IPC Namespace or privileged.
 	if hostPID || hostIPC {
@@ -265,7 +283,20 @@ func (s *Server) createSandboxContainer(ctx context.Context, ctr ctrIface.Contai
 		processLabel = ""
 	}
 
-	containerVolumes, ociMounts, err := addOCIBindMounts(ctx, mountLabel, containerConfig, specgen, s.config.RuntimeConfig.BindMountPrefix)
+	maybeRelabel := false
+	if val, present := sb.Annotations()[crioann.TrySkipVolumeSELinuxLabelAnnotation]; present && val == "true" {
+		maybeRelabel = true
+	}
+
+	skipRelabel := false
+	const superPrivilegedType = "spc_t"
+	if securityContext.SelinuxOptions.Type == superPrivilegedType || // super privileged container
+		(ctr.SandboxConfig().Linux.SecurityContext.SelinuxOptions.Type == superPrivilegedType && // super privileged pod
+			securityContext.SelinuxOptions.Type == "") {
+		skipRelabel = true
+	}
+
+	containerVolumes, ociMounts, err := addOCIBindMounts(ctx, ctr, mountLabel, s.config.RuntimeConfig.BindMountPrefix, s.config.AbsentMountSourcesToReject, maybeRelabel, skipRelabel)
 	if err != nil {
 		return nil, err
 	}
@@ -277,24 +308,16 @@ func (s *Server) createSandboxContainer(ctx context.Context, ctr ctrIface.Contai
 		return nil, err
 	}
 
-	allowDeviceAnnotations, err := s.Runtime().AllowDevicesAnnotation(sb.RuntimeHandler())
+	annotationDevices, err := device.DevicesFromAnnotation(sb.Annotations()[crioann.DevicesAnnotation])
 	if err != nil {
 		return nil, err
 	}
 
-	annotationDevices := []device.Device{}
-	if allowDeviceAnnotations {
-		annotationDevices, err = device.DevicesFromAnnotation(sb.Annotations()[crioann.DevicesAnnotation])
-		if err != nil {
-			return nil, err
-		}
-	}
-
-	if err := ctr.SpecAddDevices(configuredDevices, annotationDevices, privilegedWithoutHostDevices); err != nil {
+	if err := ctr.SpecAddDevices(configuredDevices, annotationDevices, privilegedWithoutHostDevices, s.config.DeviceOwnershipFromSecurityContext); err != nil {
 		return nil, err
 	}
 
-	labels := containerConfig.GetLabels()
+	labels := containerConfig.Labels
 
 	if err := validateLabels(labels); err != nil {
 		return nil, err
@@ -303,7 +326,7 @@ func (s *Server) createSandboxContainer(ctx context.Context, ctr ctrIface.Contai
 	// set this container's apparmor profile if it is set by sandbox
 	if s.Config().AppArmor().IsEnabled() && !ctr.Privileged() {
 		profile, err := s.Config().AppArmor().Apply(
-			securityContext.GetApparmorProfile(),
+			securityContext.ApparmorProfile,
 		)
 		if err != nil {
 			return nil, errors.Wrapf(err, "applying apparmor profile to container %s", containerID)
@@ -313,6 +336,18 @@ func (s *Server) createSandboxContainer(ctx context.Context, ctr ctrIface.Contai
 		specgen.SetProcessApparmorProfile(profile)
 	}
 
+	// Get blockio class
+	if s.Config().BlockIO().Enabled() {
+		if blockioClass, err := blockio.ContainerClassFromAnnotations(metadata.Name, containerConfig.Annotations, sb.Annotations()); blockioClass != "" && err == nil {
+			if linuxBlockIO, err := blockio.OciLinuxBlockIO(blockioClass); err == nil {
+				if specgen.Config.Linux.Resources == nil {
+					specgen.Config.Linux.Resources = &rspec.LinuxResources{}
+				}
+				specgen.Config.Linux.Resources.BlockIO = linuxBlockIO
+			}
+		}
+	}
+
 	logPath, err := ctr.LogPath(sb.LogDir())
 	if err != nil {
 		return nil, err
@@ -323,15 +358,15 @@ func (s *Server) createSandboxContainer(ctx context.Context, ctr ctrIface.Contai
 		specgen.AddProcessEnv("TERM", "xterm")
 	}
 
-	linux := containerConfig.GetLinux()
+	linux := containerConfig.Linux
 	if linux != nil {
-		resources := linux.GetResources()
+		resources := linux.Resources
 		if resources != nil {
-			specgen.SetLinuxResourcesCPUPeriod(uint64(resources.GetCpuPeriod()))
-			specgen.SetLinuxResourcesCPUQuota(resources.GetCpuQuota())
-			specgen.SetLinuxResourcesCPUShares(uint64(resources.GetCpuShares()))
+			specgen.SetLinuxResourcesCPUPeriod(uint64(resources.CPUPeriod))
+			specgen.SetLinuxResourcesCPUQuota(resources.CPUQuota)
+			specgen.SetLinuxResourcesCPUShares(uint64(resources.CPUShares))
 
-			memoryLimit := resources.GetMemoryLimitInBytes()
+			memoryLimit := resources.MemoryLimitInBytes
 			if memoryLimit != 0 {
 				if err := cgmgr.VerifyMemoryIsEnough(memoryLimit); err != nil {
 					return nil, err
@@ -342,13 +377,13 @@ func (s *Server) createSandboxContainer(ctx context.Context, ctr ctrIface.Contai
 				}
 			}
 
-			specgen.SetProcessOOMScoreAdj(int(resources.GetOomScoreAdj()))
-			specgen.SetLinuxResourcesCPUCpus(resources.GetCpusetCpus())
-			specgen.SetLinuxResourcesCPUMems(resources.GetCpusetMems())
+			specgen.SetProcessOOMScoreAdj(int(resources.OomScoreAdj))
+			specgen.SetLinuxResourcesCPUCpus(resources.CPUsetCPUs)
+			specgen.SetLinuxResourcesCPUMems(resources.CPUsetMems)
 
 			// If the kernel has no support for hugetlb, silently ignore the limits
 			if node.CgroupHasHugetlb() {
-				hugepageLimits := resources.GetHugepageLimits()
+				hugepageLimits := resources.HugepageLimits
 				for _, limit := range hugepageLimits {
 					specgen.AddLinuxResourcesHugepageLimit(limit.PageSize, limit.Limit)
 				}
@@ -360,21 +395,20 @@ func (s *Server) createSandboxContainer(ctx context.Context, ctr ctrIface.Contai
 		if ctr.Privileged() {
 			specgen.SetupPrivileged(true)
 		} else {
-			capabilities := securityContext.GetCapabilities()
+			capabilities := securityContext.Capabilities
 			// Ensure we don't get a nil pointer error if the config
 			// doesn't set any capabilities
 			if capabilities == nil {
-				capabilities = &pb.Capability{}
+				capabilities = &types.Capability{}
 			}
 			// Clear default capabilities from spec
 			specgen.ClearProcessCapabilities()
-			capabilities.AddCapabilities = append(capabilities.AddCapabilities, s.config.DefaultCapabilities...)
-			err = setupCapabilities(specgen, capabilities)
+			err = setupCapabilities(specgen, capabilities, s.config.DefaultCapabilities)
 			if err != nil {
 				return nil, err
 			}
 		}
-		specgen.SetProcessNoNewPrivileges(securityContext.GetNoNewPrivs())
+		specgen.SetProcessNoNewPrivileges(securityContext.NoNewPrivs)
 
 		if !ctr.Privileged() {
 			for _, mp := range []string{
@@ -391,9 +425,9 @@ func (s *Server) createSandboxContainer(ctx context.Context, ctr ctrIface.Contai
 			} {
 				specgen.AddLinuxMaskedPaths(mp)
 			}
-			if securityContext.GetMaskedPaths() != nil {
+			if securityContext.MaskedPaths != nil {
 				specgen.Config.Linux.MaskedPaths = nil
-				for _, path := range securityContext.GetMaskedPaths() {
+				for _, path := range securityContext.MaskedPaths {
 					specgen.AddLinuxMaskedPaths(path)
 				}
 			}
@@ -408,29 +442,36 @@ func (s *Server) createSandboxContainer(ctx context.Context, ctr ctrIface.Contai
 			} {
 				specgen.AddLinuxReadonlyPaths(rp)
 			}
-			if securityContext.GetReadonlyPaths() != nil {
+			if securityContext.ReadonlyPaths != nil {
 				specgen.Config.Linux.ReadonlyPaths = nil
-				for _, path := range securityContext.GetReadonlyPaths() {
+				for _, path := range securityContext.ReadonlyPaths {
 					specgen.AddLinuxReadonlyPaths(path)
 				}
 			}
 		}
 	}
 
+	if err := ctr.AddUnifiedResourcesFromAnnotations(sb.Annotations()); err != nil {
+		return nil, err
+	}
+
 	// Join the namespace paths for the pod sandbox container.
-	if err := configureGeneratorGivenNamespacePaths(sb.NamespacePaths(), *specgen); err != nil {
+	if err := configureGeneratorGivenNamespacePaths(sb.NamespacePaths(), specgen); err != nil {
 		return nil, errors.Wrap(err, "failed to configure namespaces in container create")
 	}
 
-	if securityContext.GetNamespaceOptions().GetPid() == pb.NamespaceMode_NODE {
+	if securityContext.NamespaceOptions.Pid == types.NamespaceModeNODE {
 		// kubernetes PodSpec specify to use Host PID namespace
 		if err := specgen.RemoveLinuxNamespace(string(rspec.PIDNamespace)); err != nil {
 			return nil, err
 		}
-	} else if securityContext.GetNamespaceOptions().GetPid() == pb.NamespaceMode_POD {
+	} else if securityContext.NamespaceOptions.Pid == types.NamespaceModePOD {
 		pidNsPath := sb.PidNsPath()
 		if pidNsPath == "" {
-			return nil, errors.New("PID namespace requested, but sandbox infra container invalid")
+			if sb.NamespaceOptions().Pid != types.NamespaceModePOD {
+				return nil, errors.New("Pod level PID namespace requested for the container, but pod sandbox was not similarly configured, and does not have an infra container")
+			}
+			return nil, errors.New("PID namespace requested, but sandbox infra container unexpectedly invalid")
 		}
 
 		if err := specgen.AddOrReplaceLinuxNamespace(string(rspec.PIDNamespace), pidNsPath); err != nil {
@@ -439,19 +480,24 @@ func (s *Server) createSandboxContainer(ctx context.Context, ctr ctrIface.Contai
 	}
 
 	// If the sandbox is configured to run in the host network, do not create a new network namespace
-	if sb.HostNetwork() {
+	if hostNet {
 		if err := specgen.RemoveLinuxNamespace(string(rspec.NetworkNamespace)); err != nil {
 			return nil, err
 		}
 
-		if !isInCRIMounts("/sys", containerConfig.GetMounts()) {
-			specgen.RemoveMount("/sys")
+		if !isInCRIMounts("/sys", containerConfig.Mounts) {
 			ctr.SpecAddMount(rspec.Mount{
 				Destination: "/sys",
 				Type:        "sysfs",
 				Source:      "sysfs",
 				Options:     []string{"nosuid", "noexec", "nodev", "ro"},
 			})
+			ctr.SpecAddMount(rspec.Mount{
+				Destination: cgroupSysFsPath,
+				Type:        "cgroup",
+				Source:      "cgroup",
+				Options:     []string{"nosuid", "noexec", "nodev", "relatime", "ro"},
+			})
 		}
 	}
 
@@ -463,7 +509,7 @@ func (s *Server) createSandboxContainer(ctx context.Context, ctr ctrIface.Contai
 			Options:     []string{"nosuid", "noexec", "nodev", "rw", "rslave"},
 		})
 		ctr.SpecAddMount(rspec.Mount{
-			Destination: "/sys/fs/cgroup",
+			Destination: cgroupSysFsPath,
 			Type:        "cgroup",
 			Source:      "cgroup",
 			Options:     []string{"nosuid", "noexec", "nodev", "rw", "relatime", "rslave"},
@@ -476,19 +522,9 @@ func (s *Server) createSandboxContainer(ctx context.Context, ctr ctrIface.Contai
 		return nil, err
 	}
 
-	processArgs, err := buildOCIProcessArgs(ctx, containerConfig, containerImageConfig)
-	if err != nil {
+	if err := ctr.SpecSetProcessArgs(containerImageConfig); err != nil {
 		return nil, err
 	}
-	specgen.SetProcessArgs(processArgs)
-
-	if strings.Contains(processArgs[0], "/sbin/init") || (filepath.Base(processArgs[0]) == "systemd") {
-		processLabel, err = selinux.InitLabel(processLabel)
-		if err != nil {
-			return nil, err
-		}
-		setupSystemd(specgen.Mounts(), *specgen)
-	}
 
 	// When running on cgroupv2, automatically add a cgroup namespace for not privileged containers.
 	if !ctr.Privileged() && node.CgroupIsV2() {
@@ -509,7 +545,7 @@ func (s *Server) createSandboxContainer(ctx context.Context, ctr ctrIface.Contai
 		options = []string{"ro"}
 	}
 	if sb.ResolvPath() != "" {
-		if err := securityLabel(sb.ResolvPath(), mountLabel, false); err != nil {
+		if err := securityLabel(sb.ResolvPath(), mountLabel, false, false); err != nil {
 			return nil, err
 		}
 		ctr.SpecAddMount(rspec.Mount{
@@ -521,7 +557,7 @@ func (s *Server) createSandboxContainer(ctx context.Context, ctr ctrIface.Contai
 	}
 
 	if sb.HostnamePath() != "" {
-		if err := securityLabel(sb.HostnamePath(), mountLabel, false); err != nil {
+		if err := securityLabel(sb.HostnamePath(), mountLabel, false, false); err != nil {
 			return nil, err
 		}
 		ctr.SpecAddMount(rspec.Mount{
@@ -532,7 +568,7 @@ func (s *Server) createSandboxContainer(ctx context.Context, ctr ctrIface.Contai
 		})
 	}
 
-	if !isInCRIMounts("/etc/hosts", containerConfig.GetMounts()) && hostNetwork(containerConfig) {
+	if !isInCRIMounts("/etc/hosts", containerConfig.Mounts) && hostNet {
 		// Only bind mount for host netns and when CRI does not give us any hosts file
 		ctr.SpecAddMount(rspec.Mount{
 			Destination: "/etc/hosts",
@@ -551,10 +587,14 @@ func (s *Server) createSandboxContainer(ctx context.Context, ctr ctrIface.Contai
 	specgen.AddProcessEnv("HOSTNAME", sb.Hostname())
 
 	created := time.Now()
-	spp := containerConfig.GetLinux().GetSecurityContext().GetSeccompProfilePath()
 	if !ctr.Privileged() {
-		if err := s.setupSeccomp(ctx, specgen, spp); err != nil {
-			return nil, err
+		if err := s.Config().Seccomp().Setup(
+			ctx,
+			specgen,
+			securityContext.Seccomp,
+			containerConfig.Linux.SecurityContext.SeccompProfilePath,
+		); err != nil {
+			return nil, errors.Wrap(err, "setup seccomp")
 		}
 	}
 
@@ -565,23 +605,39 @@ func (s *Server) createSandboxContainer(ctx context.Context, ctr ctrIface.Contai
 
 	defer func() {
 		if retErr != nil {
-			log.Infof(ctx, "createCtrLinux: stopping storage container %s", containerID)
+			log.Infof(ctx, "CreateCtrLinux: stopping storage container %s", containerID)
 			if err := s.StorageRuntimeServer().StopContainer(containerID); err != nil {
-				log.Warnf(ctx, "couldn't stop storage container: %v: %v", containerID, err)
+				log.Warnf(ctx, "Couldn't stop storage container: %v: %v", containerID, err)
 			}
 		}
 	}()
-	err = ctr.SpecAddAnnotations(sb, containerVolumes, mountPoint, containerImageConfig.Config.StopSignal, imgResult, s.config.CgroupManager().IsSystemd(), node.SystemdHasCollectMode())
+
+	// Get RDT class
+	rdtClass, err := s.Config().Rdt().ContainerClassFromAnnotations(metadata.Name, containerConfig.Annotations, sb.Annotations())
 	if err != nil {
 		return nil, err
 	}
+	if rdtClass != "" {
+		log.Debugf(ctx, "Setting RDT ClosID of container %s to %q", containerID, rdt.ResctrlPrefix+rdtClass)
+		// TODO: patch runtime-tools to support setting ClosID via a helper func similar to SetLinuxIntelRdtL3CacheSchema()
+		specgen.Config.Linux.IntelRdt = &rspec.LinuxIntelRdt{ClosID: rdt.ResctrlPrefix + rdtClass}
+	}
+
+	err = ctr.SpecAddAnnotations(ctx, sb, containerVolumes, mountPoint, containerImageConfig.Config.StopSignal, imgResult, s.config.CgroupManager().IsSystemd(), node.SystemdHasCollectMode())
+	if err != nil {
+		return nil, err
+	}
+
+	if err := s.config.Workloads.MutateSpecGivenAnnotations(ctr.Config().Metadata.Name, ctr.Spec(), sb.Annotations()); err != nil {
+		return nil, err
+	}
 
 	// First add any configured environment variables from crio config.
 	// They will get overridden if specified in the image or container config.
 	specgen.AddMultipleProcessEnv(s.Config().DefaultEnv)
 
 	// Add environment variables from image the CRI configuration
-	envs := mergeEnvs(containerImageConfig, containerConfig.GetEnvs())
+	envs := mergeEnvs(containerImageConfig, containerConfig.Envs)
 	for _, e := range envs {
 		parts := strings.SplitN(e, "=", 2)
 		specgen.AddProcessEnv(parts[0], parts[1])
@@ -617,7 +673,16 @@ func (s *Server) createSandboxContainer(ctx context.Context, ctr ctrIface.Contai
 	}
 
 	// Add secrets from the default and override mounts.conf files
-	secretMounts := secrets.SecretMounts(mountLabel, containerInfo.RunDir, s.config.DefaultMountsFile, rootless.IsRootless(), ctr.DisableFips())
+	secretMounts := subscriptions.MountsWithUIDGID(
+		mountLabel,
+		containerInfo.RunDir,
+		s.config.DefaultMountsFile,
+		containerInfo.RunDir,
+		0,
+		0,
+		rootless.IsRootless(),
+		ctr.DisableFips(),
+	)
 
 	mounts := []rspec.Mount{}
 	mounts = append(mounts, ociMounts...)
@@ -636,16 +701,24 @@ func (s *Server) createSandboxContainer(ctx context.Context, ctr ctrIface.Contai
 		ctr.SpecAddMount(rspecMount)
 	}
 
+	if ctr.WillRunSystemd() {
+		processLabel, err = selinux.InitLabel(processLabel)
+		if err != nil {
+			return nil, err
+		}
+		setupSystemd(specgen.Mounts(), *specgen)
+	}
+
 	if s.ContainerServer.Hooks != nil {
 		newAnnotations := map[string]string{}
-		for key, value := range containerConfig.GetAnnotations() {
+		for key, value := range containerConfig.Annotations {
 			newAnnotations[key] = value
 		}
 		for key, value := range sb.Annotations() {
 			newAnnotations[key] = value
 		}
 
-		if _, err := s.ContainerServer.Hooks.Hooks(specgen.Config, newAnnotations, len(containerConfig.GetMounts()) > 0); err != nil {
+		if _, err := s.ContainerServer.Hooks.Hooks(specgen.Config, newAnnotations, len(containerConfig.Mounts) > 0); err != nil {
 			return nil, err
 		}
 	}
@@ -664,7 +737,7 @@ func (s *Server) createSandboxContainer(ctx context.Context, ctr ctrIface.Contai
 		Name:    metadata.Name,
 		Attempt: metadata.Attempt,
 	}
-	ociContainer, err := oci.NewContainer(containerID, containerName, containerInfo.RunDir, logPath, labels, crioAnnotations, ctr.Config().GetAnnotations(), image, imageName, imageRef, ociMetadata, sb.ID(), containerConfig.Tty, containerConfig.Stdin, containerConfig.StdinOnce, sb.RuntimeHandler(), containerInfo.Dir, created, containerImageConfig.Config.StopSignal)
+	ociContainer, err := oci.NewContainer(containerID, containerName, containerInfo.RunDir, logPath, labels, crioAnnotations, ctr.Config().Annotations, image, imageName, imageRef, ociMetadata, sb.ID(), containerConfig.Tty, containerConfig.Stdin, containerConfig.StdinOnce, sb.RuntimeHandler(), containerInfo.Dir, created, containerImageConfig.Config.StopSignal)
 	if err != nil {
 		return nil, err
 	}
@@ -673,6 +746,7 @@ func (s *Server) createSandboxContainer(ctx context.Context, ctr ctrIface.Contai
 	specgen.SetProcessSelinuxLabel(processLabel)
 
 	ociContainer.SetIDMappings(containerIDMappings)
+	var rootPair idtools.IDPair
 	if containerIDMappings != nil {
 		s.finalizeUserMapping(specgen, containerIDMappings)
 
@@ -683,7 +757,7 @@ func (s *Server) createSandboxContainer(ctx context.Context, ctr ctrIface.Contai
 			specgen.AddLinuxGIDMapping(uint32(gidmap.HostID), uint32(gidmap.ContainerID), uint32(gidmap.Size))
 		}
 
-		rootPair := containerIDMappings.RootPair()
+		rootPair = containerIDMappings.RootPair()
 
 		pathsToChown := []string{mountPoint, containerInfo.RunDir}
 		for _, m := range secretMounts {
@@ -698,6 +772,19 @@ func (s *Server) createSandboxContainer(ctx context.Context, ctr ctrIface.Contai
 		return nil, err
 	}
 
+	if containerIDMappings == nil {
+		rootPair = idtools.IDPair{UID: 0, GID: 0}
+	}
+	// add symlink /etc/mtab to /proc/mounts allow looking for mountfiles there in the container
+	// compatible with Docker
+	mtab := filepath.Join(mountPoint, "/etc/mtab")
+	if err := idtools.MkdirAllAs(filepath.Dir(mtab), 0755, rootPair.UID, rootPair.GID); err != nil {
+		return nil, errors.Wrap(err, "error creating mtab directory")
+	}
+	if err := os.Symlink("/proc/mounts", mtab); err != nil && !os.IsExist(err) {
+		return nil, err
+	}
+
 	if os.Getenv(rootlessEnvName) != "" {
 		makeOCIConfigurationRootless(specgen)
 	}
@@ -713,7 +800,7 @@ func (s *Server) createSandboxContainer(ctx context.Context, ctr ctrIface.Contai
 
 	ociContainer.SetSpec(specgen.Config)
 	ociContainer.SetMountPoint(mountPoint)
-	ociContainer.SetSeccompProfilePath(spp)
+	ociContainer.SetSeccompProfilePath(containerConfig.Linux.SecurityContext.SeccompProfilePath)
 
 	for _, cv := range containerVolumes {
 		ociContainer.AddVolume(cv)
@@ -731,7 +818,7 @@ func setupWorkingDirectory(rootfs, mountLabel, containerCwd string) error {
 		return err
 	}
 	if mountLabel != "" {
-		if err1 := securityLabel(fp, mountLabel, false); err1 != nil {
+		if err1 := securityLabel(fp, mountLabel, false, false); err1 != nil {
 			return err1
 		}
 	}
@@ -761,10 +848,12 @@ func clearReadOnly(m *rspec.Mount) {
 	m.Options = append(m.Options, "rw")
 }
 
-func addOCIBindMounts(ctx context.Context, mountLabel string, containerConfig *pb.ContainerConfig, specgen *generate.Generator, bindMountPrefix string) ([]oci.ContainerVolume, []rspec.Mount, error) {
+func addOCIBindMounts(ctx context.Context, ctr ctrIface.Container, mountLabel, bindMountPrefix string, absentMountSourcesToReject []string, maybeRelabel, skipRelabel bool) ([]oci.ContainerVolume, []rspec.Mount, error) {
 	volumes := []oci.ContainerVolume{}
 	ociMounts := []rspec.Mount{}
-	mounts := containerConfig.GetMounts()
+	containerConfig := ctr.Config()
+	specgen := ctr.Spec()
+	mounts := containerConfig.Mounts
 
 	// Sort mounts in number of parts. This ensures that high level mounts don't
 	// shadow other mounts.
@@ -801,15 +890,17 @@ func addOCIBindMounts(ctx context.Context, mountLabel string, containerConfig *p
 		return nil, nil, err
 	}
 	for _, m := range mounts {
-		dest := m.GetContainerPath()
+		dest := m.ContainerPath
 		if dest == "" {
 			return nil, nil, fmt.Errorf("mount.ContainerPath is empty")
 		}
-
 		if m.HostPath == "" {
 			return nil, nil, fmt.Errorf("mount.HostPath is empty")
 		}
-		src := filepath.Join(bindMountPrefix, m.GetHostPath())
+		if m.HostPath == "/" && dest == "/" {
+			log.Warnf(ctx, "Configuration specifies mounting host root to the container root.  This is dangerous (especially with privileged containers) and should be avoided.")
+		}
+		src := filepath.Join(bindMountPrefix, m.HostPath)
 
 		resolvedSrc, err := resolveSymbolicLink(bindMountPrefix, src)
 		if err == nil {
@@ -817,7 +908,15 @@ func addOCIBindMounts(ctx context.Context, mountLabel string, containerConfig *p
 		} else {
 			if !os.IsNotExist(err) {
 				return nil, nil, fmt.Errorf("failed to resolve symlink %q: %v", src, err)
-			} else if err = os.MkdirAll(src, 0o755); err != nil {
+			}
+			for _, toReject := range absentMountSourcesToReject {
+				if filepath.Clean(src) == toReject {
+					// special-case /etc/hostname, as we don't want it to be created as a directory
+					// This can cause issues with node reboot.
+					return nil, nil, errors.Errorf("Cannot mount %s: path does not exist and will cause issues as a directory", toReject)
+				}
+			}
+			if err = os.MkdirAll(src, 0o755); err != nil {
 				return nil, nil, fmt.Errorf("failed to mkdir %s: %s", src, err)
 			}
 		}
@@ -829,12 +928,12 @@ func addOCIBindMounts(ctx context.Context, mountLabel string, containerConfig *p
 		options = append(options, "rbind")
 
 		// mount propagation
-		switch m.GetPropagation() {
-		case pb.MountPropagation_PROPAGATION_PRIVATE:
+		switch m.Propagation {
+		case types.MountPropagationPropagationPrivate:
 			options = append(options, "rprivate")
 			// Since default root propagation in runc is rprivate ignore
 			// setting the root propagation
-		case pb.MountPropagation_PROPAGATION_BIDIRECTIONAL:
+		case types.MountPropagationPropagationBidirectional:
 			if err := ensureShared(src, mountInfos); err != nil {
 				return nil, nil, err
 			}
@@ -842,7 +941,7 @@ func addOCIBindMounts(ctx context.Context, mountLabel string, containerConfig *p
 			if err := specgen.SetLinuxRootPropagation("rshared"); err != nil {
 				return nil, nil, err
 			}
-		case pb.MountPropagation_PROPAGATION_HOST_TO_CONTAINER:
+		case types.MountPropagationPropagationHostToContainer:
 			if err := ensureSharedOrSlave(src, mountInfos); err != nil {
 				return nil, nil, err
 			}
@@ -854,12 +953,14 @@ func addOCIBindMounts(ctx context.Context, mountLabel string, containerConfig *p
 				}
 			}
 		default:
-			log.Warnf(ctx, "unknown propagation mode for hostPath %q", m.HostPath)
+			log.Warnf(ctx, "Unknown propagation mode for hostPath %q", m.HostPath)
 			options = append(options, "rprivate")
 		}
 
 		if m.SelinuxRelabel {
-			if err := securityLabel(src, mountLabel, false); err != nil {
+			if skipRelabel {
+				logrus.Debugf("Skipping relabel for %s because of super privileged container (type: spc_t)", src)
+			} else if err := securityLabel(src, mountLabel, false, maybeRelabel); err != nil {
 				return nil, nil, err
 			}
 		}
@@ -879,7 +980,7 @@ func addOCIBindMounts(ctx context.Context, mountLabel string, containerConfig *p
 
 	if _, mountSys := mountSet["/sys"]; !mountSys {
 		m := rspec.Mount{
-			Destination: "/sys/fs/cgroup",
+			Destination: cgroupSysFsPath,
 			Type:        "cgroup",
 			Source:      "cgroup",
 			Options:     []string{"nosuid", "noexec", "nodev", "relatime", "ro"},
@@ -930,24 +1031,47 @@ func setupSystemd(mounts []rspec.Mount, g generate.Generator) {
 	}
 
 	if node.CgroupIsV2() {
-		g.RemoveMount("/sys/fs/cgroup")
+		g.RemoveMount(cgroupSysFsPath)
 
 		systemdMnt := rspec.Mount{
-			Destination: "/sys/fs/cgroup",
+			Destination: cgroupSysFsPath,
 			Type:        "cgroup",
 			Source:      "cgroup",
 			Options:     []string{"private", "rw"},
 		}
 		g.AddMount(systemdMnt)
 	} else {
-		systemdMnt := rspec.Mount{
-			Destination: "/sys/fs/cgroup/systemd",
-			Type:        "bind",
-			Source:      "/sys/fs/cgroup/systemd",
-			Options:     []string{"bind", "nodev", "noexec", "nosuid"},
+		// If the /sys/fs/cgroup is bind mounted from the host,
+		// then systemd-mode cgroup should be disabled
+		// https://bugzilla.redhat.com/show_bug.cgi?id=2064741
+		if !hasCgroupMount(g.Mounts()) {
+			systemdMnt := rspec.Mount{
+				Destination: cgroupSysFsSystemdPath,
+				Type:        "bind",
+				Source:      cgroupSysFsSystemdPath,
+				Options:     []string{"bind", "nodev", "noexec", "nosuid"},
+			}
+			g.AddMount(systemdMnt)
 		}
-		g.AddMount(systemdMnt)
 		g.AddLinuxMaskedPaths("/sys/fs/cgroup/systemd/release_agent")
 	}
 	g.AddProcessEnv("container", "crio")
 }
+
+func hasCgroupMount(mounts []rspec.Mount) bool {
+	for _, m := range mounts {
+		if (m.Destination == cgroupSysFsPath || m.Destination == "/sys/fs" || m.Destination == "/sys") && isBindMount(m.Options) {
+			return true
+		}
+	}
+	return false
+}
+
+func isBindMount(mountOptions []string) bool {
+	for _, option := range mountOptions {
+		if option == "bind" || option == "rbind" {
+			return true
+		}
+	}
+	return false
+}
