@@ -13,9 +13,9 @@ import (
 
 	cnitypes "github.com/containernetworking/cni/pkg/types"
 	current "github.com/containernetworking/cni/pkg/types/current"
-	"github.com/containers/libpod/v2/pkg/annotations"
-	"github.com/containers/libpod/v2/pkg/rootless"
-	selinux "github.com/containers/libpod/v2/pkg/selinux"
+	"github.com/containers/podman/v3/pkg/annotations"
+	"github.com/containers/podman/v3/pkg/rootless"
+	selinux "github.com/containers/podman/v3/pkg/selinux"
 	"github.com/containers/storage"
 	"github.com/containers/storage/pkg/idtools"
 	"github.com/cri-o/cri-o/internal/config/node"
@@ -28,9 +28,9 @@ import (
 	ann "github.com/cri-o/cri-o/pkg/annotations"
 	libconfig "github.com/cri-o/cri-o/pkg/config"
 	"github.com/cri-o/cri-o/pkg/sandbox"
+	"github.com/cri-o/cri-o/server/cri/types"
 	"github.com/cri-o/cri-o/utils"
 	json "github.com/json-iterator/go"
-	v1 "github.com/opencontainers/image-spec/specs-go/v1"
 	spec "github.com/opencontainers/runtime-spec/specs-go"
 	"github.com/opencontainers/runtime-tools/generate"
 	"github.com/opencontainers/selinux/go-selinux/label"
@@ -38,9 +38,7 @@ import (
 	"golang.org/x/net/context"
 	"golang.org/x/sys/unix"
 	"k8s.io/apimachinery/pkg/api/resource"
-	pb "k8s.io/cri-api/pkg/apis/runtime/v1alpha2"
-	"k8s.io/kubernetes/pkg/kubelet/leaky"
-	"k8s.io/kubernetes/pkg/kubelet/types"
+	kubeletTypes "k8s.io/kubernetes/pkg/kubelet/types"
 )
 
 // DefaultUserNSSize is the default size for the user namespace created
@@ -66,15 +64,8 @@ func addToMappingsIfMissing(ids []idtools.IDMap, id int64) []idtools.IDMap {
 	return append(ids, newMapping)
 }
 
-func (s *Server) configureSandboxIDMappings(mode string, sc *pb.LinuxSandboxSecurityContext, runtimeHandler string) (*storage.IDMappingOptions, error) {
-	// find out whether the runtime handler is configured to interpret these annotations
-	allowUsernsAnnotation, err := s.Runtime().AllowUsernsAnnotation(runtimeHandler)
-	if err != nil {
-		return nil, err
-	}
-
-	// Ignore the annotation if not explicitly set in the config file.
-	if !allowUsernsAnnotation || mode == "" {
+func (s *Server) configureSandboxIDMappings(mode string, sc *types.LinuxSandboxSecurityContext) (*storage.IDMappingOptions, error) {
+	if mode == "" {
 		// No mode specified but mappings set in the config file, let's use them.
 		if s.defaultIDMappings != nil {
 			uids := s.defaultIDMappings.UIDs()
@@ -102,7 +93,7 @@ func (s *Server) configureSandboxIDMappings(mode string, sc *pb.LinuxSandboxSecu
 	_, gidMappingsPresent := values["gidmapping"]
 	// allow these options only if running as root
 	if uidMappingsPresent || gidMappingsPresent {
-		user := sc.GetRunAsUser()
+		user := sc.RunAsUser
 		if user == nil || user.Value != 0 {
 			return nil, errors.New("cannot use uidmapping or gidmapping if not running as root")
 		}
@@ -145,43 +136,43 @@ func (s *Server) configureSandboxIDMappings(mode string, sc *pb.LinuxSandboxSecu
 			}
 			ret.AutoUserNsOpts.AdditionalGIDMappings = append(ret.AutoUserNsOpts.AdditionalGIDMappings, gids...)
 		}
-		if sc.GetRunAsUser() != nil {
+		if sc.RunAsUser != nil {
 			if keepID || mapToRoot {
 				id := 0
 				if keepID {
-					id = int(sc.GetRunAsUser().Value)
+					id = int(sc.RunAsUser.Value)
 				}
 				ret.AutoUserNsOpts.AdditionalUIDMappings = append(
 					ret.AutoUserNsOpts.AdditionalUIDMappings,
 					idtools.IDMap{
 						ContainerID: id,
-						HostID:      int(sc.GetRunAsUser().Value),
+						HostID:      int(sc.RunAsUser.Value),
 						Size:        1,
 					})
 			} else {
-				m := addToMappingsIfMissing(ret.AutoUserNsOpts.AdditionalUIDMappings, sc.GetRunAsUser().Value)
+				m := addToMappingsIfMissing(ret.AutoUserNsOpts.AdditionalUIDMappings, sc.RunAsUser.Value)
 				ret.AutoUserNsOpts.AdditionalUIDMappings = m
 			}
 		}
-		if sc.GetRunAsGroup() != nil {
+		if sc.RunAsGroup != nil {
 			if keepID || mapToRoot {
 				id := 0
 				if keepID {
-					id = int(sc.GetRunAsGroup().Value)
+					id = int(sc.RunAsGroup.Value)
 				}
 				ret.AutoUserNsOpts.AdditionalGIDMappings = append(
 					ret.AutoUserNsOpts.AdditionalGIDMappings,
 					idtools.IDMap{
 						ContainerID: id,
-						HostID:      int(sc.GetRunAsGroup().Value),
+						HostID:      int(sc.RunAsGroup.Value),
 						Size:        1,
 					})
 			} else {
-				m := addToMappingsIfMissing(ret.AutoUserNsOpts.AdditionalGIDMappings, sc.GetRunAsGroup().Value)
+				m := addToMappingsIfMissing(ret.AutoUserNsOpts.AdditionalGIDMappings, sc.RunAsGroup.Value)
 				ret.AutoUserNsOpts.AdditionalGIDMappings = m
 			}
 		}
-		for _, g := range sc.GetSupplementalGroups() {
+		for _, g := range sc.SupplementalGroups {
 			if keepID {
 				ret.AutoUserNsOpts.AdditionalGIDMappings = append(
 					ret.AutoUserNsOpts.AdditionalGIDMappings,
@@ -232,13 +223,13 @@ func (s *Server) configureSandboxIDMappings(mode string, sc *pb.LinuxSandboxSecu
 			}
 		}
 		// make sure the specified users are part of the namespace
-		if sc.GetRunAsUser() != nil {
-			uids = addToMappingsIfMissing(uids, sc.GetRunAsUser().Value)
+		if sc.RunAsUser != nil {
+			uids = addToMappingsIfMissing(uids, sc.RunAsUser.Value)
 		}
-		if sc.GetRunAsGroup() != nil {
-			gids = addToMappingsIfMissing(gids, sc.GetRunAsGroup().Value)
+		if sc.RunAsGroup != nil {
+			gids = addToMappingsIfMissing(gids, sc.RunAsGroup.Value)
 		}
-		for _, g := range sc.GetSupplementalGroups() {
+		for _, g := range sc.SupplementalGroups {
 			gids = addToMappingsIfMissing(gids, g)
 		}
 
@@ -259,16 +250,6 @@ func (s *Server) getSandboxIDMappings(sb *libsandbox.Sandbox) (*idtools.IDMappin
 		return nil, nil
 	}
 
-	// find out whether the runtime handler is configured to interpret these annotations
-	allowUsernsAnnotation, err := s.Runtime().AllowUsernsAnnotation(sb.RuntimeHandler())
-	if err != nil {
-		return nil, err
-	}
-
-	// Ignore the annotation if not explicitly set in the config file.
-	if s.defaultIDMappings == nil && !allowUsernsAnnotation {
-		return nil, nil
-	}
 	if ic == nil {
 		return nil, errors.Errorf("infra container not found")
 	}
@@ -287,23 +268,23 @@ func (s *Server) getSandboxIDMappings(sb *libsandbox.Sandbox) (*idtools.IDMappin
 	return mappings, nil
 }
 
-func (s *Server) runPodSandbox(ctx context.Context, req *pb.RunPodSandboxRequest) (resp *pb.RunPodSandboxResponse, retErr error) {
+func (s *Server) runPodSandbox(ctx context.Context, req *types.RunPodSandboxRequest) (resp *types.RunPodSandboxResponse, retErr error) {
 	s.updateLock.RLock()
 	defer s.updateLock.RUnlock()
 
-	sbox := sandbox.New(ctx)
-	if err := sbox.SetConfig(req.GetConfig()); err != nil {
+	sbox := sandbox.New()
+	if err := sbox.SetConfig(req.Config); err != nil {
 		return nil, errors.Wrap(err, "setting sandbox config")
 	}
 
 	pathsToChown := []string{}
 
 	// we need to fill in the container name, as it is not present in the request. Luckily, it is a constant.
-	log.Infof(ctx, "Running pod sandbox: %s%s", translateLabelsToDescription(sbox.Config().GetLabels()), leaky.PodInfraContainerName)
+	log.Infof(ctx, "Running pod sandbox: %s%s", translateLabelsToDescription(sbox.Config().Labels), types.InfraContainerName)
 
-	kubeName := sbox.Config().GetMetadata().GetName()
-	namespace := sbox.Config().GetMetadata().GetNamespace()
-	attempt := sbox.Config().GetMetadata().GetAttempt()
+	kubeName := sbox.Config().Metadata.Name
+	namespace := sbox.Config().Metadata.Namespace
+	attempt := sbox.Config().Metadata.Attempt
 
 	if err := sbox.SetNameAndID(); err != nil {
 		return nil, errors.Wrap(err, "setting pod sandbox name and id")
@@ -321,13 +302,36 @@ func (s *Server) runPodSandbox(ctx context.Context, req *pb.RunPodSandboxRequest
 	}()
 
 	if _, err := s.ReservePodName(sbox.ID(), sbox.Name()); err != nil {
+		reservedID, getErr := s.PodIDForName(sbox.Name())
+		if getErr != nil {
+			return nil, errors.Wrapf(getErr, "Failed to get ID of pod with reserved name (%s), after failing to reserve name with %v", sbox.Name(), getErr)
+		}
+		// if we're able to find the sandbox, and it's created, this is actually a duplicate request
+		// from a client that does not behave like the kubelet (like crictl)
+		if reservedSbox := s.GetSandbox(reservedID); reservedSbox != nil && reservedSbox.Created() {
+			return nil, err
+		}
 		cachedID, resourceErr := s.getResourceOrWait(ctx, sbox.Name(), "sandbox")
 		if resourceErr == nil {
-			return &pb.RunPodSandboxResponse{PodSandboxId: cachedID}, nil
+			return &types.RunPodSandboxResponse{PodSandboxID: cachedID}, nil
 		}
 		return nil, errors.Wrapf(err, resourceErr.Error())
 	}
 
+	securityContext := sbox.Config().Linux.SecurityContext
+	hostNetwork := securityContext.NamespaceOptions.Network == types.NamespaceModeNODE
+
+	if err := s.config.CNIPluginReadyOrError(); err != nil && !hostNetwork {
+		// if the cni plugin isn't ready yet, we should wait until it is
+		// before proceeding
+		watcher := s.config.CNIPluginAddWatcher()
+		log.Infof(ctx, "CNI plugin not ready. Waiting to create %s as it is not host network", sbox.Name())
+		if ready := <-watcher; !ready {
+			return nil, errors.Wrapf(err, "server shutdown before network was ready")
+		}
+		log.Infof(ctx, "CNI plugin is now ready. Continuing to create %s", sbox.Name())
+	}
+
 	description := fmt.Sprintf("runSandbox: releasing pod sandbox name: %s", sbox.Name())
 	resourceCleaner.Add(ctx, description, func() error {
 		log.Infof(ctx, description)
@@ -335,17 +339,21 @@ func (s *Server) runPodSandbox(ctx context.Context, req *pb.RunPodSandboxRequest
 		return nil
 	})
 
-	kubeAnnotations := sbox.Config().GetAnnotations()
-
-	usernsMode := kubeAnnotations[ann.UsernsModeAnnotation]
-
 	// validate the runtime handler
 	runtimeHandler, err := s.runtimeHandler(req)
 	if err != nil {
 		return nil, err
 	}
 
-	idMappingsOptions, err := s.configureSandboxIDMappings(usernsMode, sbox.Config().GetLinux().GetSecurityContext(), runtimeHandler)
+	if err := s.Runtime().FilterDisallowedAnnotations(runtimeHandler, sbox.Config().Annotations); err != nil {
+		return nil, errors.Wrap(err, "filter disallowed annotations")
+	}
+
+	kubeAnnotations := sbox.Config().Annotations
+
+	usernsMode := kubeAnnotations[ann.UsernsModeAnnotation]
+
+	idMappingsOptions, err := s.configureSandboxIDMappings(usernsMode, sbox.Config().Linux.SecurityContext)
 	if err != nil {
 		return nil, err
 	}
@@ -362,8 +370,7 @@ func (s *Server) runPodSandbox(ctx context.Context, req *pb.RunPodSandboxRequest
 	})
 
 	var labelOptions []string
-	securityContext := sbox.Config().GetLinux().GetSecurityContext()
-	selinuxConfig := securityContext.GetSelinuxOptions()
+	selinuxConfig := securityContext.SelinuxOptions
 	if selinuxConfig != nil {
 		labelOptions = utils.GetLabelOptions(selinuxConfig)
 	}
@@ -377,7 +384,7 @@ func (s *Server) runPodSandbox(ctx context.Context, req *pb.RunPodSandboxRequest
 		"",
 		containerName,
 		kubeName,
-		sbox.Config().GetMetadata().GetUid(),
+		sbox.Config().Metadata.UID,
 		namespace,
 		attempt,
 		idMappingsOptions,
@@ -399,13 +406,13 @@ func (s *Server) runPodSandbox(ctx context.Context, req *pb.RunPodSandboxRequest
 		log.Infof(ctx, description)
 		err2 := s.StorageRuntimeServer().DeleteContainer(sbox.ID())
 		if err2 != nil {
-			log.Warnf(ctx, "could not cleanup pod sandbox %q: %v", sbox.ID(), err2)
+			log.Warnf(ctx, "Could not cleanup pod sandbox %q: %v", sbox.ID(), err2)
 		}
 		return err2
 	})
 
 	// set log directory
-	logDir := sbox.Config().GetLogDirectory()
+	logDir := sbox.Config().LogDirectory
 	if logDir == "" {
 		logDir = filepath.Join(s.config.LogDir, sbox.ID())
 	}
@@ -423,67 +430,21 @@ func (s *Server) runPodSandbox(ctx context.Context, req *pb.RunPodSandboxRequest
 	}
 
 	// TODO: factor generating/updating the spec into something other projects can vendor
-
-	// creates a spec Generator with the default spec.
-	g, err := generate.New("linux")
-	if err != nil {
+	if err := sbox.InitInfraContainer(&s.config, &podContainer); err != nil {
 		return nil, err
 	}
-	g.HostSpecific = true
-	g.ClearProcessRlimits()
-
-	for _, u := range s.config.Ulimits() {
-		g.AddProcessRlimits(u.Name, u.Hard, u.Soft)
-	}
-
-	// setup defaults for the pod sandbox
-	g.SetRootReadonly(true)
-
-	pauseCommand, err := PauseCommand(s.Config(), podContainer.Config)
-	if err != nil {
-		return nil, err
-	}
-	g.SetProcessArgs(pauseCommand)
-
-	// set DNS options
-	var resolvPath string
-	if sbox.Config().GetDnsConfig() != nil {
-		dnsServers := sbox.Config().GetDnsConfig().Servers
-		dnsSearches := sbox.Config().GetDnsConfig().Searches
-		dnsOptions := sbox.Config().GetDnsConfig().Options
-		resolvPath = fmt.Sprintf("%s/resolv.conf", podContainer.RunDir)
-		err = parseDNSOptions(dnsServers, dnsSearches, dnsOptions, resolvPath)
-		if err != nil {
-			err1 := removeFile(resolvPath)
-			if err1 != nil {
-				return nil, fmt.Errorf("%v; failed to remove %s: %v", err, resolvPath, err1)
-			}
-			return nil, err
-		}
-		if err := label.Relabel(resolvPath, mountLabel, false); err != nil && !errors.Is(err, unix.ENOTSUP) {
-			if err1 := removeFile(resolvPath); err1 != nil {
-				return nil, fmt.Errorf("%v; failed to remove %s: %v", err, resolvPath, err1)
-			}
-		}
-		mnt := spec.Mount{
-			Type:        "bind",
-			Source:      resolvPath,
-			Destination: "/etc/resolv.conf",
-			Options:     []string{"ro", "bind", "nodev", "nosuid", "noexec"},
-		}
-		pathsToChown = append(pathsToChown, resolvPath)
-		g.AddMount(mnt)
-	}
+	g := sbox.Spec()
+	pathsToChown = append(pathsToChown, sbox.ResolvPath())
 
 	// add metadata
-	metadata := sbox.Config().GetMetadata()
+	metadata := sbox.Config().Metadata
 	metadataJSON, err := json.Marshal(metadata)
 	if err != nil {
 		return nil, err
 	}
 
 	// add labels
-	labels := sbox.Config().GetLabels()
+	labels := sbox.Config().Labels
 
 	if err := validateLabels(labels); err != nil {
 		return nil, err
@@ -491,7 +452,7 @@ func (s *Server) runPodSandbox(ctx context.Context, req *pb.RunPodSandboxRequest
 
 	// Add special container name label for the infra container
 	if labels != nil {
-		labels[types.KubernetesContainerNameLabel] = leaky.PodInfraContainerName
+		labels[kubeletTypes.KubernetesContainerNameLabel] = types.InfraContainerName
 	}
 	labelsJSON, err := json.Marshal(labels)
 	if err != nil {
@@ -505,22 +466,19 @@ func (s *Server) runPodSandbox(ctx context.Context, req *pb.RunPodSandboxRequest
 	}
 
 	// Add capabilities from crio.conf if default_capabilities is defined
-	capabilities := &pb.Capability{}
-	if s.config.DefaultCapabilities != nil {
-		g.ClearProcessCapabilities()
-		capabilities.AddCapabilities = append(capabilities.AddCapabilities, s.config.DefaultCapabilities...)
-	}
-	if err := setupCapabilities(&g, capabilities); err != nil {
+	capabilities := &types.Capability{}
+	g.ClearProcessCapabilities()
+	if err := setupCapabilities(g, capabilities, s.config.DefaultCapabilities); err != nil {
 		return nil, err
 	}
 
-	nsOptsJSON, err := json.Marshal(securityContext.GetNamespaceOptions())
+	nsOptsJSON, err := json.Marshal(securityContext.NamespaceOptions)
 	if err != nil {
 		return nil, err
 	}
 
-	hostIPC := securityContext.GetNamespaceOptions().GetIpc() == pb.NamespaceMode_NODE
-	hostPID := securityContext.GetNamespaceOptions().GetPid() == pb.NamespaceMode_NODE
+	hostIPC := securityContext.NamespaceOptions.Ipc == types.NamespaceModeNODE
+	hostPID := securityContext.NamespaceOptions.Pid == types.NamespaceModeNODE
 
 	// Don't use SELinux separation with Host Pid or IPC Namespace or privileged.
 	if hostPID || hostIPC {
@@ -538,18 +496,12 @@ func (s *Server) runPodSandbox(ctx context.Context, req *pb.RunPodSandboxRequest
 		shmPath = libsandbox.DevShmPath
 	} else {
 		shmSize := int64(libsandbox.DefaultShmSize)
-		allowShmSizeAnnotations, err := s.Runtime().AllowShmSizeAnnotation(runtimeHandler)
-		if err != nil {
-			return nil, fmt.Errorf("failed to allow shmsize annotation")
-		}
-		if allowShmSizeAnnotations {
-			if shmSizeStr, ok := kubeAnnotations[ann.ShmSizeAnnotation]; ok {
-				quantity, err := resource.ParseQuantity(shmSizeStr)
-				if err != nil {
-					return nil, fmt.Errorf("failed to parse shm size '%s': %v", shmSizeStr, err)
-				}
-				shmSize = quantity.Value()
+		if shmSizeStr, ok := kubeAnnotations[ann.ShmSizeAnnotation]; ok {
+			quantity, err := resource.ParseQuantity(shmSizeStr)
+			if err != nil {
+				return nil, fmt.Errorf("failed to parse shm size '%s': %v", shmSizeStr, err)
 			}
+			shmSize = quantity.Value()
 		}
 		shmPath, err = setupShm(podContainer.RunDir, mountLabel, shmSize)
 		if err != nil {
@@ -561,7 +513,7 @@ func (s *Server) runPodSandbox(ctx context.Context, req *pb.RunPodSandboxRequest
 			log.Infof(ctx, description)
 			err2 := unix.Unmount(shmPath, unix.MNT_DETACH)
 			if err2 != nil {
-				log.Warnf(ctx, "failed to unmount shm for sandbox: %v", err2)
+				log.Warnf(ctx, "Failed to unmount shm for sandbox: %v", err2)
 			}
 			return err2
 		})
@@ -594,7 +546,7 @@ func (s *Server) runPodSandbox(ctx context.Context, req *pb.RunPodSandboxRequest
 			if strings.Contains(err2.Error(), noSuchID) {
 				return nil
 			}
-			log.Warnf(ctx, "Could not delete ctr id %s from idIndex: %v", sbox.ID(), err2)
+			log.Warnf(ctx, "Could not delete ctr id %s from idIndex", sbox.ID())
 		}
 		return err2
 	})
@@ -607,8 +559,6 @@ func (s *Server) runPodSandbox(ctx context.Context, req *pb.RunPodSandboxRequest
 		return nil, err
 	}
 
-	hostNetwork := securityContext.GetNamespaceOptions().GetNetwork() == pb.NamespaceMode_NODE
-
 	hostname, err := getHostname(sbox.ID(), sbox.Config().Hostname, hostNetwork)
 	if err != nil {
 		return nil, err
@@ -629,7 +579,7 @@ func (s *Server) runPodSandbox(ctx context.Context, req *pb.RunPodSandboxRequest
 	g.AddAnnotation(annotations.ShmPath, shmPath)
 	g.AddAnnotation(annotations.PrivilegedRuntime, fmt.Sprintf("%v", privileged))
 	g.AddAnnotation(annotations.RuntimeHandler, runtimeHandler)
-	g.AddAnnotation(annotations.ResolvPath, resolvPath)
+	g.AddAnnotation(annotations.ResolvPath, sbox.ResolvPath())
 	g.AddAnnotation(annotations.HostName, hostname)
 	g.AddAnnotation(annotations.NamespaceOptions, string(nsOptsJSON))
 	g.AddAnnotation(annotations.KubeName, kubeName)
@@ -647,14 +597,14 @@ func (s *Server) runPodSandbox(ctx context.Context, req *pb.RunPodSandboxRequest
 	created := time.Now()
 	g.AddAnnotation(annotations.Created, created.Format(time.RFC3339Nano))
 
-	portMappings := convertPortMappings(sbox.Config().GetPortMappings())
+	portMappings := convertPortMappings(sbox.Config().PortMappings)
 	portMappingsJSON, err := json.Marshal(portMappings)
 	if err != nil {
 		return nil, err
 	}
 	g.AddAnnotation(annotations.PortMappings, string(portMappingsJSON))
 
-	cgroupParent, cgroupPath, err := s.config.CgroupManager().SandboxCgroupPath(sbox.Config().GetLinux().GetCgroupParent(), sbox.ID())
+	cgroupParent, cgroupPath, err := s.config.CgroupManager().SandboxCgroupPath(sbox.Config().Linux.CgroupParent, sbox.ID())
 	if err != nil {
 		return nil, err
 	}
@@ -676,12 +626,12 @@ func (s *Server) runPodSandbox(ctx context.Context, req *pb.RunPodSandboxRequest
 	}
 
 	sbMetadata := &libsandbox.Metadata{
-		Name:      metadata.GetName(),
-		UID:       metadata.GetUid(),
-		Namespace: metadata.GetNamespace(),
-		Attempt:   metadata.GetAttempt(),
+		Name:      metadata.Name,
+		UID:       metadata.UID,
+		Namespace: metadata.Namespace,
+		Attempt:   metadata.Attempt,
 	}
-	sb, err := libsandbox.New(sbox.ID(), namespace, sbox.Name(), kubeName, logDir, labels, kubeAnnotations, processLabel, mountLabel, sbMetadata, shmPath, cgroupParent, privileged, runtimeHandler, resolvPath, hostname, portMappings, hostNetwork, created, usernsMode)
+	sb, err := libsandbox.New(sbox.ID(), namespace, sbox.Name(), kubeName, logDir, labels, kubeAnnotations, processLabel, mountLabel, sbMetadata, shmPath, cgroupParent, privileged, runtimeHandler, sbox.ResolvPath(), hostname, portMappings, hostNetwork, created, usernsMode)
 	if err != nil {
 		return nil, err
 	}
@@ -694,7 +644,7 @@ func (s *Server) runPodSandbox(ctx context.Context, req *pb.RunPodSandboxRequest
 		log.Infof(ctx, description)
 		err := s.removeSandbox(sbox.ID())
 		if err != nil {
-			log.Warnf(ctx, "could not remove pod sandbox: %v", err)
+			log.Warnf(ctx, "Could not remove pod sandbox: %v", err)
 		}
 		return err
 	})
@@ -708,10 +658,7 @@ func (s *Server) runPodSandbox(ctx context.Context, req *pb.RunPodSandboxRequest
 		log.Infof(ctx, description)
 		err := s.PodIDIndex().Delete(sbox.ID())
 		if err != nil {
-			if strings.Contains(err.Error(), noSuchID) {
-				return nil
-			}
-			log.Warnf(ctx, "Could not delete pod id %s from idIndex: %v", sbox.ID(), err)
+			log.Warnf(ctx, "Could not delete pod id %s from idIndex", sbox.ID())
 		}
 		return err
 	})
@@ -724,7 +671,7 @@ func (s *Server) runPodSandbox(ctx context.Context, req *pb.RunPodSandboxRequest
 	}
 
 	// Add default sysctls given in crio.conf
-	sysctls := s.configureGeneratorForSysctls(ctx, g, hostNetwork, hostIPC, req.GetConfig().GetLinux().GetSysctls())
+	sysctls := s.configureGeneratorForSysctls(ctx, g, hostNetwork, hostIPC, req.Config.Linux.Sysctls)
 
 	// set up namespaces
 	nsCleanupFuncs, err := s.configureGeneratorForSandboxNamespaces(hostNetwork, hostIPC, hostPID, sandboxIDMappings, sysctls, sb, g)
@@ -734,7 +681,7 @@ func (s *Server) runPodSandbox(ctx context.Context, req *pb.RunPodSandboxRequest
 		log.Infof(ctx, description)
 		for idx := range nsCleanupFuncs {
 			if err2 := nsCleanupFuncs[idx](); err2 != nil {
-				log.Infof(ctx, "runSandbox: failed to cleanup namespace: %s", err2.Error())
+				log.Infof(ctx, "RunSandbox: failed to cleanup namespace: %s", err2.Error())
 				return err2
 			}
 		}
@@ -749,36 +696,34 @@ func (s *Server) runPodSandbox(ctx context.Context, req *pb.RunPodSandboxRequest
 	var ips []string
 	var result cnitypes.Result
 
-	if s.config.ManageNSLifecycle {
-		ips, result, err = s.networkStart(ctx, sb)
+	ips, result, err = s.networkStart(ctx, sb)
+	if err != nil {
+		resourceCleaner.Add(ctx, nsCleanupDescription, nsCleanupFunc)
+		return nil, err
+	}
+	description = fmt.Sprintf("runSandbox: stopping network for sandbox %s", sb.ID())
+	resourceCleaner.Add(ctx, description, func() error {
+		log.Infof(ctx, description)
+		// use a new context to prevent an expired context from preventing a stop
+		if err := s.networkStop(context.Background(), sb); err != nil {
+			log.Errorf(ctx, "Error stopping network on cleanup: %v", err)
+			return err
+		}
+
+		// Now that we've succeeded in stopping the network, cleanup namespaces
+		log.Infof(ctx, nsCleanupDescription)
+		return nsCleanupFunc()
+	})
+	if result != nil {
+		resultCurrent, err := current.NewResultFromResult(result)
 		if err != nil {
-			resourceCleaner.Add(ctx, nsCleanupDescription, nsCleanupFunc)
 			return nil, err
 		}
-		description = fmt.Sprintf("runSandbox: stopping network for sandbox %s", sb.ID())
-		resourceCleaner.Add(ctx, description, func() error {
-			log.Infof(ctx, description)
-			// use a new context to prevent an expired context from preventing a stop
-			if err := s.networkStop(context.Background(), sb); err != nil {
-				log.Errorf(ctx, "error stopping network on cleanup: %v", err)
-				return err
-			}
-
-			// Now that we've succeeded in stopping the network, cleanup namespaces
-			log.Infof(ctx, nsCleanupDescription)
-			return nsCleanupFunc()
-		})
-		if result != nil {
-			resultCurrent, err := current.NewResultFromResult(result)
-			if err != nil {
-				return nil, err
-			}
-			cniResultJSON, err := json.Marshal(resultCurrent)
-			if err != nil {
-				return nil, err
-			}
-			g.AddAnnotation(annotations.CNIResult, string(cniResultJSON))
+		cniResultJSON, err := json.Marshal(resultCurrent)
+		if err != nil {
+			return nil, err
 		}
+		g.AddAnnotation(annotations.CNIResult, string(cniResultJSON))
 	}
 
 	// Set OOM score adjust of the infra container to be very low
@@ -803,7 +748,7 @@ func (s *Server) runPodSandbox(ctx context.Context, req *pb.RunPodSandboxRequest
 		log.Infof(ctx, description)
 		err2 := s.StorageRuntimeServer().StopContainer(sbox.ID())
 		if err2 != nil {
-			log.Warnf(ctx, "could not stop storage container: %v: %v", sbox.ID(), err2)
+			log.Warnf(ctx, "Could not stop storage container: %v: %v", sbox.ID(), err2)
 		}
 		return err2
 	})
@@ -828,7 +773,7 @@ func (s *Server) runPodSandbox(ctx context.Context, req *pb.RunPodSandboxRequest
 	sb.AddHostnamePath(hostnamePath)
 
 	if sandboxIDMappings != nil {
-		if securityContext.GetNamespaceOptions().GetIpc() == pb.NamespaceMode_NODE {
+		if securityContext.NamespaceOptions.Ipc == types.NamespaceModeNODE {
 			g.RemoveMount("/dev/mqueue")
 			mqueue := spec.Mount{
 				Type:        "bind",
@@ -849,7 +794,7 @@ func (s *Server) runPodSandbox(ctx context.Context, req *pb.RunPodSandboxRequest
 			}
 			g.AddMount(sysMnt)
 		}
-		if securityContext.GetNamespaceOptions().GetPid() == pb.NamespaceMode_NODE {
+		if securityContext.NamespaceOptions.Pid == types.NamespaceModeNODE {
 			g.RemoveMount("/proc")
 			proc := spec.Mount{
 				Type:        "bind",
@@ -869,25 +814,19 @@ func (s *Server) runPodSandbox(ctx context.Context, req *pb.RunPodSandboxRequest
 	g.SetRootPath(mountPoint)
 
 	if os.Getenv(rootlessEnvName) != "" {
-		makeOCIConfigurationRootless(&g)
+		makeOCIConfigurationRootless(g)
 	}
 
-	namespaceOpts := securityContext.GetNamespaceOptions()
-	sb.SetNamespaceOptions(
-		&libsandbox.NamespaceOption{
-			Network:  libsandbox.NamespaceMode(namespaceOpts.GetNetwork()),
-			Pid:      libsandbox.NamespaceMode(namespaceOpts.GetPid()),
-			Ipc:      libsandbox.NamespaceMode(namespaceOpts.GetIpc()),
-			TargetID: namespaceOpts.GetTargetId(),
-		},
-	)
+	sb.SetNamespaceOptions(securityContext.NamespaceOptions)
 
-	spp := securityContext.GetSeccompProfilePath() // nolint:staticcheck
-	g.AddAnnotation(annotations.SeccompProfilePath, spp)
-	sb.SetSeccompProfilePath(spp)
+	seccompProfilePath := securityContext.SeccompProfilePath
+	g.AddAnnotation(annotations.SeccompProfilePath, seccompProfilePath)
+	sb.SetSeccompProfilePath(seccompProfilePath)
 	if !privileged {
-		if err := s.setupSeccomp(ctx, &g, spp); err != nil {
-			return nil, err
+		if err := s.Config().Seccomp().Setup(
+			ctx, g, securityContext.Seccomp, seccompProfilePath,
+		); err != nil {
+			return nil, errors.Wrap(err, "setup seccomp")
 		}
 	}
 
@@ -904,7 +843,7 @@ func (s *Server) runPodSandbox(ctx context.Context, req *pb.RunPodSandboxRequest
 	var container *oci.Container
 	// In the case of kernel separated containers, we need the infra container to create the VM for the pod
 	if sb.NeedsInfra(s.config.DropInfraCtr) || podIsKernelSeparated {
-		log.Debugf(ctx, "keeping infra container for pod %s", sbox.ID())
+		log.Debugf(ctx, "Keeping infra container for pod %s", sbox.ID())
 		container, err = oci.NewContainer(sbox.ID(), containerName, podContainer.RunDir, logPath, labels, g.Config.Annotations, kubeAnnotations, s.config.PauseImage, "", "", nil, sbox.ID(), false, false, false, runtimeHandler, podContainer.Dir, created, podContainer.Config.Config.StopSignal)
 		if err != nil {
 			return nil, err
@@ -918,15 +857,17 @@ func (s *Server) runPodSandbox(ctx context.Context, req *pb.RunPodSandboxRequest
 			}
 			g.SetProcessSelinuxLabel(processLabel)
 		}
-
-		container.SetMountPoint(mountPoint)
-
-		container.SetSpec(g.Config)
 	} else {
-		log.Debugf(ctx, "dropping infra container for pod %s", sbox.ID())
-		container = oci.NewSpoofedContainer(sbox.ID(), containerName, labels, created, podContainer.RunDir)
+		log.Debugf(ctx, "Dropping infra container for pod %s", sbox.ID())
+		container = oci.NewSpoofedContainer(sbox.ID(), containerName, labels, sbox.ID(), created, podContainer.RunDir)
 		g.AddAnnotation(ann.SpoofedContainer, "true")
+		if err := s.config.CgroupManager().CreateSandboxCgroup(cgroupParent, sbox.ID()); err != nil {
+			return nil, errors.Wrapf(err, "create dropped infra %s cgroup", sbox.ID())
+		}
 	}
+	container.SetMountPoint(mountPoint)
+	container.SetSpec(g.Config)
+
 	// needed for getSandboxIDMappings()
 	container.SetIDMappings(sandboxIDMappings)
 
@@ -961,11 +902,11 @@ func (s *Server) runPodSandbox(ctx context.Context, req *pb.RunPodSandboxRequest
 		}
 	}
 
-	if err := s.createContainerPlatform(container, sb.CgroupParent(), sandboxIDMappings); err != nil {
+	if err := s.createContainerPlatform(ctx, container, sb.CgroupParent(), sandboxIDMappings); err != nil {
 		return nil, err
 	}
 
-	if err := s.Runtime().StartContainer(container); err != nil {
+	if err := s.Runtime().StartContainer(ctx, container); err != nil {
 		return nil, err
 	}
 
@@ -977,37 +918,21 @@ func (s *Server) runPodSandbox(ctx context.Context, req *pb.RunPodSandboxRequest
 			return errors.Errorf("failed to stop container for removal")
 		}
 
-		log.Infof(ctx, "runSandbox: deleting container %s", container.ID())
-		if err2 := s.Runtime().DeleteContainer(container); err2 != nil {
-			log.Warnf(ctx, "failed to delete container %s in pod sandbox %s: %v", container.Name(), sb.ID(), err2)
+		log.Infof(ctx, "RunSandbox: deleting container %s", container.ID())
+		if err2 := s.Runtime().DeleteContainer(ctx, container); err2 != nil {
+			log.Warnf(ctx, "Failed to delete container %s in pod sandbox %s: %v", container.Name(), sb.ID(), err2)
 			return err2
 		}
-		log.Infof(ctx, "runSandbox: writing container %s state to disk", container.ID())
-		if err2 := s.ContainerStateToDisk(container); err2 != nil {
-			log.Warnf(ctx, "failed to write container state %s in pod sandbox %s: %v", container.Name(), sb.ID(), err2)
+		log.Infof(ctx, "RunSandbox: writing container %s state to disk", container.ID())
+		if err2 := s.ContainerStateToDisk(ctx, container); err2 != nil {
+			log.Warnf(ctx, "Failed to write container state %s in pod sandbox %s: %v", container.Name(), sb.ID(), err2)
 			return err2
 		}
 		return nil
 	})
 
-	if err := s.ContainerStateToDisk(container); err != nil {
-		log.Warnf(ctx, "unable to write containers %s state to disk: %v", container.ID(), err)
-	}
-
-	if !s.config.ManageNSLifecycle {
-		ips, _, err = s.networkStart(ctx, sb)
-		if err != nil {
-			return nil, err
-		}
-		description = fmt.Sprintf("runSandbox: stopping network for sandbox %s when not manage NS", sb.ID())
-		resourceCleaner.Add(ctx, description, func() error {
-			log.Infof(ctx, description)
-			err2 := s.networkStop(context.Background(), sb)
-			if err2 != nil {
-				log.Errorf(ctx, "error stopping network on cleanup: %v", err2)
-			}
-			return err2
-		})
+	if err := s.ContainerStateToDisk(ctx, container); err != nil {
+		log.Warnf(ctx, "Unable to write containers %s state to disk: %v", container.ID(), err)
 	}
 
 	for idx, ip := range ips {
@@ -1017,15 +942,15 @@ func (s *Server) runPodSandbox(ctx context.Context, req *pb.RunPodSandboxRequest
 
 	if isContextError(ctx.Err()) {
 		if err := s.resourceStore.Put(sbox.Name(), sb, resourceCleaner); err != nil {
-			log.Errorf(ctx, "runSandbox: failed to save progress of sandbox %s: %v", sbox.ID(), err)
+			log.Errorf(ctx, "RunSandbox: failed to save progress of sandbox %s: %v", sbox.ID(), err)
 		}
-		log.Infof(ctx, "runSandbox: context was either canceled or the deadline was exceeded: %v", ctx.Err())
+		log.Infof(ctx, "RunSandbox: context was either canceled or the deadline was exceeded: %v", ctx.Err())
 		return nil, ctx.Err()
 	}
 	sb.SetCreated()
 
 	log.Infof(ctx, "Ran pod sandbox %s with infra container: %s", container.ID(), container.Description())
-	resp = &pb.RunPodSandboxResponse{PodSandboxId: sbox.ID()}
+	resp = &types.RunPodSandboxResponse{PodSandboxID: sbox.ID()}
 	return resp, nil
 }
 
@@ -1046,36 +971,11 @@ func setupShm(podSandboxRunDir, mountLabel string, shmSize int64) (shmPath strin
 	return shmPath, nil
 }
 
-// PauseCommand returns the pause command for the provided image configuration.
-func PauseCommand(cfg *libconfig.Config, image *v1.Image) ([]string, error) {
-	if cfg == nil {
-		return nil, fmt.Errorf("provided configuration is nil")
-	}
-
-	// This has been explicitly set by the user, since the configuration
-	// default is `/pause`
-	if cfg.PauseCommand == "" {
-		if image == nil ||
-			(len(image.Config.Entrypoint) == 0 && len(image.Config.Cmd) == 0) {
-			return nil, fmt.Errorf(
-				"unable to run pause image %q: %s",
-				cfg.PauseImage,
-				"neither Cmd nor Entrypoint specified",
-			)
-		}
-		cmd := []string{}
-		cmd = append(cmd, image.Config.Entrypoint...)
-		cmd = append(cmd, image.Config.Cmd...)
-		return cmd, nil
-	}
-	return []string{cfg.PauseCommand}, nil
-}
-
-func (s *Server) configureGeneratorForSysctls(ctx context.Context, g generate.Generator, hostNetwork, hostIPC bool, sysctls map[string]string) map[string]string {
+func (s *Server) configureGeneratorForSysctls(ctx context.Context, g *generate.Generator, hostNetwork, hostIPC bool, sysctls map[string]string) map[string]string {
 	sysctlsToReturn := make(map[string]string)
 	defaultSysctls, err := s.config.RuntimeConfig.Sysctls()
 	if err != nil {
-		log.Warnf(ctx, "sysctls invalid: %v", err)
+		log.Warnf(ctx, "Sysctls invalid: %v", err)
 	}
 
 	for _, sysctl := range defaultSysctls {
@@ -1105,54 +1005,48 @@ func (s *Server) configureGeneratorForSysctls(ctx context.Context, g generate.Ge
 // as well as whether CRI-O should be managing the namespace lifecycle.
 // it returns a slice of cleanup funcs, all of which are the respective NamespaceRemove() for the sandbox.
 // The caller should defer the cleanup funcs if there is an error, to make sure each namespace we are managing is properly cleaned up.
-func (s *Server) configureGeneratorForSandboxNamespaces(hostNetwork, hostIPC, hostPID bool, idMappings *idtools.IDMappings, sysctls map[string]string, sb *libsandbox.Sandbox, g generate.Generator) (cleanupFuncs []func() error, retErr error) {
-	managedNamespaces := make([]nsmgr.NSType, 0, 3)
-	if hostNetwork {
-		if err := g.RemoveLinuxNamespace(string(spec.NetworkNamespace)); err != nil {
-			return nil, err
-		}
-	} else if s.config.ManageNSLifecycle {
-		managedNamespaces = append(managedNamespaces, nsmgr.NETNS)
-	}
-
-	if hostIPC {
-		if err := g.RemoveLinuxNamespace(string(spec.IPCNamespace)); err != nil {
-			return nil, err
-		}
-	} else if s.config.ManageNSLifecycle {
-		managedNamespaces = append(managedNamespaces, nsmgr.IPCNS)
-	}
-
-	if idMappings == nil {
-		if err := g.RemoveLinuxNamespace(string(spec.UserNamespace)); err != nil {
-			return nil, err
-		}
-	} else if s.config.ManageNSLifecycle {
-		managedNamespaces = append(managedNamespaces, nsmgr.USERNS)
-	}
-
+func (s *Server) configureGeneratorForSandboxNamespaces(hostNetwork, hostIPC, hostPID bool, idMappings *idtools.IDMappings, sysctls map[string]string, sb *libsandbox.Sandbox, g *generate.Generator) (cleanupFuncs []func() error, retErr error) {
 	// Since we need a process to hold open the PID namespace, CRI-O can't manage the NS lifecycle
 	if hostPID {
 		if err := g.RemoveLinuxNamespace(string(spec.PIDNamespace)); err != nil {
 			return nil, err
 		}
 	}
+	namespaceConfig := &nsmgr.PodNamespacesConfig{
+		Sysctls:    sysctls,
+		IDMappings: idMappings,
+		Namespaces: []*nsmgr.PodNamespaceConfig{
+			{
+				Type: nsmgr.IPCNS,
+				Host: hostIPC,
+			},
+			{
+				Type: nsmgr.NETNS,
+				Host: hostNetwork,
+			},
+			{
+				Type: nsmgr.UTSNS, // there is no option for host UTSNS
+			},
+		},
+	}
+	if idMappings != nil {
+		namespaceConfig.Namespaces = append(namespaceConfig.Namespaces, &nsmgr.PodNamespaceConfig{
+			Type: nsmgr.USERNS,
+		})
+	}
 
-	// There's no option to set hostUTS
-	if s.config.ManageNSLifecycle {
-		managedNamespaces = append(managedNamespaces, nsmgr.UTSNS)
+	// now that we've configured the namespaces we're sharing, create them
+	namespaces, err := s.config.NamespaceManager().NewPodNamespaces(namespaceConfig)
+	if err != nil {
+		return nil, err
+	}
 
-		// now that we've configured the namespaces we're sharing, tell sandbox to configure them
-		managedNamespaces, err := sb.CreateManagedNamespaces(managedNamespaces, idMappings, sysctls, &s.config)
-		if err != nil {
-			return nil, err
-		}
+	sb.AddManagedNamespaces(namespaces)
 
-		cleanupFuncs = append(cleanupFuncs, sb.RemoveManagedNamespaces)
+	cleanupFuncs = append(cleanupFuncs, sb.RemoveManagedNamespaces)
 
-		if err := configureGeneratorGivenNamespacePaths(managedNamespaces, g); err != nil {
-			return cleanupFuncs, err
-		}
+	if err := configureGeneratorGivenNamespacePaths(sb.NamespacePaths(), g); err != nil {
+		return cleanupFuncs, err
 	}
 
 	return cleanupFuncs, nil
@@ -1160,7 +1054,7 @@ func (s *Server) configureGeneratorForSandboxNamespaces(hostNetwork, hostIPC, ho
 
 // configureGeneratorGivenNamespacePaths takes a map of nsType -> nsPath. It configures the generator
 // to add or replace the defaults to these paths
-func configureGeneratorGivenNamespacePaths(managedNamespaces []*libsandbox.ManagedNamespace, g generate.Generator) error {
+func configureGeneratorGivenNamespacePaths(managedNamespaces []*libsandbox.ManagedNamespace, g *generate.Generator) error {
 	typeToSpec := map[nsmgr.NSType]spec.LinuxNamespaceType{
 		nsmgr.IPCNS:  spec.IPCNamespace,
 		nsmgr.NETNS:  spec.NetworkNamespace,
