@@ -4,13 +4,11 @@ import (
 	"context"
 
 	"github.com/cri-o/cri-o/internal/storage"
-	"github.com/cri-o/cri-o/pkg/config"
-	"github.com/cri-o/cri-o/server"
+	"github.com/cri-o/cri-o/server/cri/types"
 	"github.com/golang/mock/gomock"
 	. "github.com/onsi/ginkgo"
 	. "github.com/onsi/gomega"
 	v1 "github.com/opencontainers/image-spec/specs-go/v1"
-	pb "k8s.io/cri-api/pkg/apis/runtime/v1alpha2"
 )
 
 // The actual test suite
@@ -28,10 +26,6 @@ var _ = t.Describe("RunPodSandbox", func() {
 		// cyclomatic complexity and test it separately
 		It("should fail when container creation errors", func() {
 			// Given
-			// when we ManageNSLifecycle, we do networking setup before we do container creation
-			// mocking the networking setup blows up complexity of this test, which is really
-			// not testing the behavior of managing ns lifecycle. Override default for this test
-			sut.SetManageNSLifecycle(false)
 			gomock.InOrder(
 				runtimeServerMock.EXPECT().CreatePodSandbox(gomock.Any(),
 					gomock.Any(), gomock.Any(), gomock.Any(), gomock.Any(),
@@ -46,27 +40,23 @@ var _ = t.Describe("RunPodSandbox", func() {
 					Return(storage.RuntimeContainerMetadata{}, nil),
 				runtimeServerMock.EXPECT().SetContainerMetadata(gomock.Any(),
 					gomock.Any()).Return(nil),
-				runtimeServerMock.EXPECT().StartContainer(gomock.Any()).
-					Return("", nil),
-				runtimeServerMock.EXPECT().StopContainer(gomock.Any()).
-					Return(nil),
 				runtimeServerMock.EXPECT().DeleteContainer(gomock.Any()).
 					Return(nil),
 			)
 
 			// When
 			response, err := sut.RunPodSandbox(context.Background(),
-				&pb.RunPodSandboxRequest{Config: &pb.PodSandboxConfig{
-					Metadata: &pb.PodSandboxMetadata{
+				&types.RunPodSandboxRequest{Config: &types.PodSandboxConfig{
+					Metadata: &types.PodSandboxMetadata{
 						Name:      "name",
 						Namespace: "default",
-						Uid:       "uid",
+						UID:       "uid",
 					},
 					LogDirectory: "/tmp",
-					Linux: &pb.LinuxPodSandboxConfig{
-						SecurityContext: &pb.LinuxSandboxSecurityContext{
-							NamespaceOptions: &pb.NamespaceOption{
-								Ipc: pb.NamespaceMode_NODE,
+					Linux: &types.LinuxPodSandboxConfig{
+						SecurityContext: &types.LinuxSandboxSecurityContext{
+							NamespaceOptions: &types.NamespaceOption{
+								Ipc: types.NamespaceModeNODE,
 							},
 						},
 					},
@@ -81,7 +71,7 @@ var _ = t.Describe("RunPodSandbox", func() {
 			// Given
 			// When
 			response, err := sut.RunPodSandbox(context.Background(),
-				&pb.RunPodSandboxRequest{Config: &pb.PodSandboxConfig{}})
+				&types.RunPodSandboxRequest{Config: &types.PodSandboxConfig{}})
 
 			// Then
 			Expect(err).NotTo(BeNil())
@@ -92,8 +82,8 @@ var _ = t.Describe("RunPodSandbox", func() {
 			// Given
 			// When
 			response, err := sut.RunPodSandbox(context.Background(),
-				&pb.RunPodSandboxRequest{Config: &pb.PodSandboxConfig{
-					Metadata: &pb.PodSandboxMetadata{},
+				&types.RunPodSandboxRequest{Config: &types.PodSandboxConfig{
+					Metadata: &types.PodSandboxMetadata{},
 				}})
 
 			// Then
@@ -105,8 +95,8 @@ var _ = t.Describe("RunPodSandbox", func() {
 			// Given
 			// When
 			response, err := sut.RunPodSandbox(context.Background(),
-				&pb.RunPodSandboxRequest{Config: &pb.PodSandboxConfig{
-					Metadata: &pb.PodSandboxMetadata{
+				&types.RunPodSandboxRequest{Config: &types.PodSandboxConfig{
+					Metadata: &types.PodSandboxMetadata{
 						Name: "name",
 					},
 				}})
@@ -131,11 +121,14 @@ var _ = t.Describe("RunPodSandbox", func() {
 
 			// When
 			response, err := sut.RunPodSandbox(context.Background(),
-				&pb.RunPodSandboxRequest{Config: &pb.PodSandboxConfig{
-					Metadata: &pb.PodSandboxMetadata{
+				&types.RunPodSandboxRequest{Config: &types.PodSandboxConfig{
+					Metadata: &types.PodSandboxMetadata{
 						Name:      "name",
 						Namespace: "default",
-						Uid:       "uid",
+						UID:       "uid",
+					},
+					Linux: &types.LinuxPodSandboxConfig{
+						SecurityContext: types.NewLinuxSandboxSecurityContext(),
 					},
 				}})
 
@@ -144,93 +137,4 @@ var _ = t.Describe("RunPodSandbox", func() {
 			Expect(response).To(BeNil())
 		})
 	})
-
-	t.Describe("PauseCommand", func() {
-		var cfg *config.Config
-
-		BeforeEach(func() {
-			// Given
-			var err error
-			cfg, err = config.DefaultConfig()
-			Expect(err).To(BeNil())
-		})
-
-		It("should succeed with default config", func() {
-			// When
-			res, err := server.PauseCommand(cfg, nil)
-
-			// Then
-			Expect(err).To(BeNil())
-			Expect(res).To(Equal([]string{sut.Config().PauseCommand}))
-		})
-
-		It("should succeed with Entrypoint", func() {
-			// Given
-			cfg.PauseCommand = ""
-			entrypoint := []string{"/custom-pause"}
-			image := &v1.Image{Config: v1.ImageConfig{Entrypoint: entrypoint}}
-
-			// When
-			res, err := server.PauseCommand(cfg, image)
-
-			// Then
-			Expect(err).To(BeNil())
-			Expect(res).To(Equal(entrypoint))
-		})
-
-		It("should succeed with Cmd", func() {
-			// Given
-			cfg.PauseCommand = ""
-			cmd := []string{"some-cmd"}
-			image := &v1.Image{Config: v1.ImageConfig{Cmd: cmd}}
-
-			// When
-			res, err := server.PauseCommand(cfg, image)
-
-			// Then
-			Expect(err).To(BeNil())
-			Expect(res).To(Equal(cmd))
-		})
-
-		It("should succeed with Entrypoint and Cmd", func() {
-			// Given
-			cfg.PauseCommand = ""
-			entrypoint := "/custom-pause"
-			cmd := "some-cmd"
-			image := &v1.Image{Config: v1.ImageConfig{
-				Entrypoint: []string{entrypoint},
-				Cmd:        []string{cmd},
-			}}
-
-			// When
-			res, err := server.PauseCommand(cfg, image)
-
-			// Then
-			Expect(err).To(BeNil())
-			Expect(res).To(HaveLen(2))
-			Expect(res[0]).To(Equal(entrypoint))
-			Expect(res[1]).To(Equal(cmd))
-		})
-
-		It("should fail if config is nil", func() {
-			// When
-			res, err := server.PauseCommand(nil, nil)
-
-			// Then
-			Expect(err).NotTo(BeNil())
-			Expect(res).To(BeNil())
-		})
-
-		It("should fail if image config is nil", func() {
-			// Given
-			cfg.PauseCommand = ""
-
-			// When
-			res, err := server.PauseCommand(cfg, nil)
-
-			// Then
-			Expect(err).NotTo(BeNil())
-			Expect(res).To(BeNil())
-		})
-	})
 })
